@@ -1,9 +1,8 @@
-#include "getopt.h"
-
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <fcntl.h>
 #include <termios.h>
 #include <errno.h>
@@ -25,6 +24,7 @@
 #include "shared/debug.h"
 #include "shared/report.h"
 #include "shared/configfile.h"
+#include "getopt.h"
 
 #include "batt.h"
 #include "chrono.h"
@@ -70,16 +70,20 @@ static int process_configfile(char *cfgfile);
 #if !defined(SYSCONFDIR)
 # define SYSCONFDIR	"/etc"
 #endif
+#if !defined(PIDFILEDIR)
+# define PIDFILEDIR	"/var/run"
+#endif
 
 #define UNSET_INT		-1
 #define UNSET_STR		"\01"
 #define DEFAULT_SERVER		"127.0.0.1"
 #define DEFAULT_CONFIGFILE	SYSCONFDIR "/lcdproc.conf"
+#define DEFAULT_PIDFILE		PIDFILEDIR "/lcdproc.pid"
 #define DEFAULT_REPORTDEST	RPT_DEST_STDERR
 #define DEFAULT_REPORTLEVEL	RPT_WARNING
 
-/* list of modes to run */
-mode sequence[] =
+/* list of screen modes to run */
+ScreenMode sequence[] =
 {
 	// flags default ACTIVE will run by default
 	// longname    which on  off inv  timer   flags
@@ -112,6 +116,8 @@ int foreground 	= FALSE;
 static int report_level = UNSET_INT;
 static int report_dest 	= UNSET_INT;
 char *configfile = NULL;
+char *pidfile = NULL;
+int pidfile_written = FALSE;
 char *displayname = NULL;
 
 
@@ -188,6 +194,7 @@ main(int argc, char **argv)
 	signal(SIGINT, exit_program);	// Ctrl-C
 	signal(SIGTERM, exit_program);	// "regular" kill
 	signal(SIGHUP, exit_program);	// kill -HUP
+	signal(SIGPIPE, exit_program);	// write to closed socket
 	signal(SIGKILL, exit_program);	// kill -9 [cannot be trapped; but ...]
 
 	/* No error output from getopt */
@@ -243,7 +250,7 @@ main(int argc, char **argv)
 	/* Read config file*/
 	cfgresult = process_configfile(configfile);
 	if (cfgresult < 0) {
-		fprintf(stderr, "Error reading config file");
+		fprintf(stderr, "Error reading config file\n");
 		exit(EXIT_FAILURE);
 	}	
 
@@ -269,7 +276,7 @@ main(int argc, char **argv)
 			int state = (*argv[i] == '!') ? 0 : 1;
 			char *name = (state) ? argv[i] : argv[i]+1;
 			int shortname = (strlen(name) == 1) ? name[0] : '\0';
-			// debug: fprintf(stderr, "%s%s\n", (state) ? "" : "!", name);
+			// debug: fprintf(stderr, "%s%s", (state) ? "" : "!", name);
 			int found = set_mode(shortname, name, state);
 
 			if (!found) {
@@ -303,9 +310,23 @@ main(int argc, char **argv)
 
 	if (foreground != TRUE) {
 		if (daemon(1,0) != 0) {
-			fprintf(stderr, "Error: daemonize failed");
+			fprintf(stderr, "Error: daemonize failed\n");
 			return(EXIT_FAILURE);
 		}
+
+		if (pidfile != NULL) {
+			FILE *pidf = fopen(pidfile, "w");
+
+			if (pidf) {
+				fprintf(pidf, "%d\n", (int) getpid());
+				fclose(pidf);
+				pidfile_written = TRUE;
+			} else {
+				fprintf(stderr, "Error creating pidfile %s: %s\n",
+					pidfile, strerror(errno));
+				return(EXIT_FAILURE);
+			}
+		}
 	}	
 
 	// Init the status gatherers...
@@ -314,10 +335,8 @@ main(int argc, char **argv)
 	// And spew stuff!
 	main_loop();
 
-	// Clean up
+	// Clean up & exit
 	exit_program(EXIT_SUCCESS);
-
-	return(0);
 }
 
 
@@ -372,6 +391,9 @@ process_configfile(char *configfile)
 	if (foreground != TRUE) {
 		foreground = config_get_bool(progname, "Foreground", 0, FALSE);
 	}
+	if (pidfile == NULL) {
+		pidfile = strdup(config_get_string(progname, "PidFile", 0, DEFAULT_PIDFILE));
+	}
 	if (islow < 0) {
 		islow = config_get_int(progname, "Delay", 0, -1);
 	}
@@ -403,7 +425,7 @@ HelpScreen (int exit_state)
 	fprintf(stderr,
 		"lcdproc - LCDproc system status information viewer\n"
 		"\n"
-		"Copyright (c) 1999-2006 Scott Scriven, William Ferrell, and misc. contributors.\n"
+		"Copyright (c) 1999-2009 Scott Scriven, William Ferrell, and misc. contributors.\n"
 		"This program is released under the terms of the GNU General Public License.\n"
 		"\n"
 		"Usage: lcdproc [<options>] [<screens> ...]\n"
@@ -451,10 +473,11 @@ exit_program(int val)
 	 */
 	eyebox_clear();
 #endif
-	//printf("exit program\n");
 	Quit = 1;
 	sock_close(sock);
 	mode_close();
+	if ((foreground != TRUE) && (pidfile != NULL) && (pidfile_written == TRUE))
+		unlink(pidfile);
 	exit(val);
 }
 
@@ -554,7 +577,7 @@ main_loop(void)
 								for (j = 0; sequence[j].which; j++) {
 									if (sequence[j].which == argv[1][0]) {
 										sequence[j].flags |= VISIBLE;
-										//debug(RPT_DEBUG, "Listen %s\n", argv[1]);
+										//debug(RPT_DEBUG, "Listen %s", argv[1]);
 									}
 								}
 							}
@@ -562,12 +585,12 @@ main_loop(void)
 								for (j = 0; sequence[j].which; j++) {
 									if (sequence[j].which == argv[1][0]) {
 										sequence[j].flags &= ~VISIBLE;
-										//debug(RPT_DEBUG, "Ignore %s\n", argv[1]);
+										//debug(RPT_DEBUG, "Ignore %s", argv[1]);
 									}
 								}
 							}
 							else if (0 == strcmp(argv[0], "key")) {
-								debug(RPT_DEBUG, "Key %s\n", argv[1]);
+								debug(RPT_DEBUG, "Key %s", argv[1]);
 							}
 #ifdef LCDPROC_MENUS
 							else if (0 == strcmp(argv[0], "menuevent")) {
