@@ -41,7 +41,7 @@ mode_close(void)
 }
 
 int
-update_screen(mode *m, int display)
+update_screen(ScreenMode *m, int display)
 {
 	static int status = -1;
 	int old_status = status;
@@ -84,6 +84,10 @@ update_screen(mode *m, int display)
 int
 credit_screen(int rep, int display, int *flags_ptr)
 {
+	/*
+	 * List of persons who contributed to LCDproc.
+	 * Keep in sync with CREDITS file (ordered by appearance)
+	 */
 	const char *contributors[] = {
 		"William Ferrell",
 		"Scott Scriven",
@@ -131,6 +135,24 @@ credit_screen(int rep, int display, int *flags_ptr)
 		"Anthony J. Mirabella",
 		"Cedric Tessier",
 		"John Sanders",
+		"Eric Pooch",
+		"Benjamin Wiedmann",
+		"Frank Jepsen",
+		"Karsten Festag",
+		"Gatewood Green",
+		"Dave Platt",
+		"Nicu Pavel",
+		"Daryl Fonseca-Holt",
+		"Thien Vu",
+		"Thomas Jarosch",
+		"Christian Jodar",
+		"Mariusz Bialonczyk",
+		"Jack Cleaver",
+		"Aron Parsons",
+		"Malte Poeggel",
+		"Dean Harding",
+		"Christian Leuschen",
+		"Jonathan Kyler",
 		NULL
 	};
 	int contr_num = 0;
