@@ -7,7 +7,7 @@
 int mode_init(void);
 void mode_close(void);
 
-int update_screen(mode * m, int display);
+int update_screen(ScreenMode *m, int display);
 
 int credit_screen(int rep, int display, int *flags_ptr);
 
