@@ -33,7 +33,7 @@ static int read_connect_string(void);
 static int split(char *str, char delim, char *parts[], int maxparts);
 
 
-int connect_and_setup(void)
+int setup_connection(void)
 {
 	char buf[200];
 	int i;
@@ -83,6 +83,14 @@ int connect_and_setup(void)
 }
 
 
+int teardown_connection(void)
+{
+	sock_close(sock);
+
+	return 0;
+}
+
+
 static int read_connect_string(void)
 {
 	char buf[8192];
