@@ -1,3 +1,7 @@
+/** \file server/drivers/CFontz633io.c
+ * IO routines for the drivers \c CFontzPacket and \c CFontz633.
+ */
+
 /* interfacing routines for the CrystalFontz CFA-631, CFA-633 and CFA-635 LCDs
 
   ===========================================================================
@@ -46,8 +50,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
-
-#include "config.h"
+#include <errno.h>	/* only required for debugging */
 
 #if defined(HAVE_SYS_SELECT_H)
 # include <sys/select.h>
@@ -56,7 +59,12 @@
 # include <sys/types.h>
 #endif /* defined(HAVE_SYS_SELECT_H) */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "CFontz633io.h"
+#include "report.h"
 
 
 #define TRY_AGAIN 0
@@ -75,7 +83,9 @@ static void send_packet(int fd, COMMAND_PACKET *out, COMMAND_PACKET *in);
 static int  get_crc(unsigned char *buf, int len, int seed);
 static int  test_packet(int fd, unsigned char response, COMMAND_PACKET *in);
 static int  check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length);
+#ifdef DEBUG
 static void print_packet(COMMAND_PACKET *packet);
+#endif
 
 
 /* global variables */
@@ -90,18 +100,27 @@ ReceiveBuffer receivebuffer;
  * It is just a small fifo of unsigned char.
  */
 
-/** initialize/empty key ring by resetting its read & write pointers */
+/**
+ * Initialize/empty key ring by resetting its read & write pointers.
+ * \param kr  Pointer to KeyRing.
+ */
 void EmptyKeyRing(KeyRing *kr)
 {
 	kr->head = kr->tail = 0;
 }
 
 
-/** add byte to key ring; return success (byte added) / failure (key ring is full) */
+/**
+ * Add byte to key ring.
+ * \param kr   Pointer to KeyRing.
+ * \param key  Key byte to add.
+ * \retval 1  Success (byte added).
+ * \retval 0  Failure (key ring is full).
+ */
 int AddKeyToKeyRing(KeyRing *kr, unsigned char key)
 {
 	if (((kr->head + 1) % KEYRINGSIZE) != (kr->tail % KEYRINGSIZE)) {
-		/* fprintf(stderr, "We add key: %d\n", key); */
+		//debug(RPT_DEBUG, "%s: add key: %d", __FUNCTION__, key);
 
 	        kr->contents[kr->head % KEYRINGSIZE] = key;
   		kr->head = (kr->head + 1) % KEYRINGSIZE;
@@ -113,7 +132,12 @@ int AddKeyToKeyRing(KeyRing *kr, unsigned char key)
 }
 
 
-/** get byte from key ring (or '\\0' if key ring is empty) */
+/**
+ * Get byte from key ring.
+ * \param kr  Pointer to KeyRing.
+ * \retval retval  Byte from KeyRing.
+ * \retval '\0'    Failure (key ring is empty).
+ */
 unsigned char GetKeyFromKeyRing(KeyRing *kr)
 {
 	unsigned char retval = '\0';
@@ -125,13 +149,21 @@ unsigned char GetKeyFromKeyRing(KeyRing *kr)
 	        kr->tail = (kr->tail + 1) % KEYRINGSIZE;
 	}
 
-	/*  if (retval) fprintf(stderr, "We remove key: %d\n", retval); */
+	//if (retval)
+	//	debug(RPT_DEBUG, "%s: remove key: %d", __FUNCTION__, retval);
+
 	return retval;
 }
 
 
 
-/** send message with arguments to the given handle */
+/**
+ * Send message with arguments to the given handle.
+ * \param fd    File handle to write to.
+ * \param msg   Command byte to write.
+ * \param len   Length (in bytes) of data following.
+ * \param data  Pointer to command argument data.
+ */
 void send_bytes_message(int fd, unsigned char msg, int len, unsigned char *data)
 {
 	COMMAND_PACKET out;
@@ -146,7 +178,12 @@ void send_bytes_message(int fd, unsigned char msg, int len, unsigned char *data)
 }
 
 
-/** send message with one byte argument to the given handle */
+/**
+ * Send message with one byte argument to the given handle.
+ * \param fd     File handle to write to.
+ * \param msg    Command byte to write.
+ * \param value  Command argument.
+ */
 void send_onebyte_message(int fd, unsigned char msg, unsigned char value)
 {
 	COMMAND_PACKET out;
@@ -161,7 +198,11 @@ void send_onebyte_message(int fd, unsigned char msg, unsigned char value)
 }
 
 
-/** send message without data to the given handle */
+/**
+ * Send message without arguments to the given handle.
+ * \param fd    File handle to write to.
+ * \param msg   Command byte to write.
+ */
 void send_zerobyte_message(int fd, unsigned char msg)
 {
 	COMMAND_PACKET out;
@@ -175,7 +216,12 @@ void send_zerobyte_message(int fd, unsigned char msg)
 }
 
 
-/** send out to the given handle; calc & send CRC when doing so */
+/**
+ * Send out to the given handle; calc & send CRC when doing so.
+ * \param fd    File handle to write to.
+ * \param out   Pointer to COMMAND_PACKET structure to write.
+ * \param in    Pointer to COMMAND_PACKET structure to read after write.
+ */
 static void
 send_packet(int fd, COMMAND_PACKET *out, COMMAND_PACKET *in)
 {
@@ -200,7 +246,13 @@ send_packet(int fd, COMMAND_PACKET *out, COMMAND_PACKET *in)
 }
 
 
-/** calculate CRC over given buffer with given length */
+/**
+ * Calculate CRC over given buffer with given length.
+ * \param buf   Byte buffer.
+ * \param len   Length of byte buffer.
+ * \param seed  CRC seed value.
+ * \return  CRC calulated.
+ */
 /* According to the "Painless Guide to CRC error dectectin algorithmsi
  * (http://www.repairfaq.org/filipg/LINK/F_crc_v3.html) this is the table driven
  * implementation of a CRC with the following parameters:
@@ -269,14 +321,22 @@ get_crc(unsigned char *buf, int len, int seed)
 /*           ^ tail              ^ head               */
 
 
-/** initialize/empty receive buffer by resetting its pointers */
+/**
+ * Initialize/empty receive buffer by resetting its pointers.
+ * \param rb  Pointer to ReceiveBuffer structure.
+ */
 void EmptyReceiveBuffer(ReceiveBuffer *rb)
 {
 	rb->head = rb->tail = rb->peek = 0;
 }
 
 
-/** read given number of bytes from given file handle into receive buffer */
+/**
+ * Read given number of bytes from given file handle into receive buffer.
+ * \param rb      Pointer to ReceiveBuffer structure.
+ * \param fd      File handle to read from.
+ * \param number  Max. number of bytes to read from file handle.
+ */
 void SyncReceiveBuffer(ReceiveBuffer *rb, int fd, unsigned int number)
 {
 	unsigned char buffer[MAX_DATA_LENGTH];
@@ -302,31 +362,34 @@ void SyncReceiveBuffer(ReceiveBuffer *rb, int fd, unsigned int number)
 	BytesRead = read(fd, buffer, number);
 
 	if (BytesRead == -1) {
-		/* this shouldnot happen with the select() above */
-		/* fprintf(stderr, "~~~Problem reading: %s .\n", strerror(errno)); */
+		/* this should not happen with the select() above */
+		//debug(RPT_WARNING, "%s: ~~~Problem reading: %s", __FUNCTION__, strerror(errno));
 	}
 	else {
 		int	i;
 
-		/* fprintf(stderr, "Read %d Bytes:", BytesRead); */
+		//debug(RPT_DEBUG, "%s: read %d bytes:", __FUNCTION__, BytesRead);
 
 		/* wrap write pointer to the receive buffer */
 		rb->head %= RECEIVEBUFFERSIZE;
 
 		/* store the bytes read */
 		for (i = 0; i < BytesRead; i++) {
-			/* fprintf(stderr, " %02x", buffer[i]); */
+			//debug(RPT_DEBUG, "%s: reading byte %02x", __FUNCTION__, buffer[i]);
 			rb->contents[rb->head] = buffer[i];
 
 			/* increment write pointer (wrap if needed) */
 			rb->head = (rb->head + 1) % RECEIVEBUFFERSIZE;
 		}
-		/* fprintf(stderr, "\n"); */
 	}
 }
 
 
-/** return number of bytes available for reading in receive buffer */
+/**
+ * Get number of bytes available for reading in receive buffer.
+ * \param rb  Pointer to ReceiveBuffer structure.
+ * \return  Number of bytes available in receive buffer.
+ */
 int BytesAvail(ReceiveBuffer *rb)
 {
 	int avail_bytes = rb->head - rb->tail;
@@ -338,7 +401,12 @@ int BytesAvail(ReceiveBuffer *rb)
 }
 
 
-/** get next byte from receive buffer (return '\\0' if buffer is empty) */
+/**
+ * Get next byte from receive buffer.
+ * \param rb  Pointer to ReceiveBuffer structure.
+ * \retval return_byte  Next byte in receive buffer.
+ * \retval '\0'         Failure (receive buffer is empty).
+ */
 unsigned char GetByte(ReceiveBuffer *rb)
 {
 	unsigned char return_byte = '\0';
@@ -359,7 +427,11 @@ unsigned char GetByte(ReceiveBuffer *rb)
 }
 
 
-/** return number of bytes available for peeking in receive buffer */
+/**
+ * Return number of bytes available for peeking in receive buffer.
+ * \param rb  Pointer to ReceiveBuffer structure.
+ * \return  Number of bytes available for peeking in receive buffer.
+ */
 int PeekBytesAvail(ReceiveBuffer *rb)
 {
 	int avail_bytes = rb->head - rb->peek;
@@ -371,21 +443,32 @@ int PeekBytesAvail(ReceiveBuffer *rb)
 }
 
 
-/** sync peek pointer with read pointer */
+/**
+ * Sync peek pointer with read pointer.
+ * \param rb  Pointer to ReceiveBuffer structure.
+ */
 void SyncPeekPointer(ReceiveBuffer *rb)
 {
 	rb->peek = rb->tail;
 }
 
 
-/** accept peeked data by syncing the read pointer to the peek pointer */
+/**
+ * Accept peeked data by syncing the read pointer to the peek pointer.
+ * \param rb  Pointer to ReceiveBuffer structure.
+ */
 void AcceptPeekedData(ReceiveBuffer *rb)
 {
 	rb->tail = rb->peek;
 }
 
 
-/** peek next byte from receive buffer (return '\\0' if buffer is empty) */
+/**
+ * Peek next byte from receive buffer.
+ * \param rb  Pointer to ReceiveBuffer structure.
+ * \retval return_byte  Next byte in receive buffer.
+ * \retval '\0'         Failure (receive buffer is empty).
+ */
 unsigned char PeekByte(ReceiveBuffer *rb)
 {
 	unsigned char return_byte = '\0';
@@ -407,6 +490,14 @@ unsigned char PeekByte(ReceiveBuffer *rb)
 
 
 
+/**
+ * Check for a response packet and try to identify it.
+ * \param fd        File handle to read from.
+ * \param response  Expected response command.
+ * \param in        Pointer to COMMAND_PACKET structure to write the response to.
+ * \retval 1  Expected response received.
+ * \retval 0  Expected response not received.
+ */
 /* I should use the value GIVE_UP and not reenter if there is no extra
  * byte read from the serial port
  */ 
@@ -462,12 +553,14 @@ test_packet(int fd, unsigned char response, COMMAND_PACKET *in)
  */
 
 
-/* Let's return
- * O if we have no message but we should try again immediatly
- * 1 if we have a message correctly identified
- * 2 if we have no message and we should not retry until new input
- * So a loop should run as long as we have no 0
- * If we have a 2 we should avoid comming back there.
+/**
+ * Check for a packet to read.
+ * \param fd        File handle to read from.
+ * \param in        Pointer to COMMAND_PACKET structure to write the response to.
+ * \param expected_length  Expected response length.
+ * \retval GIVE_UP    No message and we should not retry until new input.
+ * \retval TRY_AGAIN  No message but we should try again immediately.
+ * \retval GOOD_MSG   Message correctly identified.
  */
 static int
 check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length)
@@ -480,7 +573,7 @@ check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length)
 	//First off, there must be at least 4 bytes available in the input stream
 	//for there to be a valid command in it (command, length, no data, CRC).
 	if (BytesAvail(&receivebuffer) < 4) {
-		/* fprintf(stderr, "Not enough bytes available for even the smallest message.\n"); */
+		//debug(RPT_INFO, "%s: not enough bytes available for even the smallest message", __FUNCTION__);
 		return(GIVE_UP); /* We don't need to retry before more byte are received */
 	}
 
@@ -494,7 +587,7 @@ check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length)
 	if (MAX_COMMAND < (0x3F & in->command)) {
 		/* Throw out one byte of garbage. Next pass through should re-sync. */
 		GetByte(&receivebuffer);
-		/* fprintf(stderr, "###: Unknown command.\n"); */
+		//debug(RPT_INFO, "%s: unknown command", __FUNCTION__);
 		return(TRY_AGAIN);
 	}
 
@@ -505,7 +598,7 @@ check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length)
   	if (MAX_DATA_LENGTH < in->data_length) {
 		//Throw out one byte of garbage. Next pass through should re-sync.
 		GetByte(&receivebuffer);
-		/* fprintf(stderr, "###: Too long packet: %d.\n", in->data_length); */
+		//debug(RPT_INFO, "%s: too long packet: %d", __FUNCTION__, in->data_length);
 		return(TRY_AGAIN);
 	}
 
@@ -514,7 +607,7 @@ check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length)
 	if ((int) PeekBytesAvail(&receivebuffer) < (in->data_length + 2)) {
 		//It looked like a valid start of a packet, but it does not look
 		//like the complete packet has been received yet.
-		/* fprintf(stderr, "Not enough read to check the complete message.\n"); */
+		//debug(RPT_INFO, "%s: not enough read to check the complete message", __FUNCTION__);
 		return(GIVE_UP); /* Let's not return until more byte are available */
 	}
 
@@ -534,7 +627,7 @@ check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length)
 		//This is a good packet. Remove the packet from the serial buffer.
 		AcceptPeekedData(&receivebuffer);
 		//Let our caller know that incoming_command has good stuff in it.
-		/* print_packet(&outgoing_response); */
+		/* print_packet(in); */
 
 		return(GOOD_MSG);
 	}
@@ -543,12 +636,13 @@ check_for_packet(int fd, COMMAND_PACKET *in, unsigned char expected_length)
 	* Next pass through should re-sync.
 	*/
 	GetByte(&receivebuffer);
-	/* fprintf(stderr, "###: Wrong CheckSum. computed/real %04x:%04x \n",
-  		   testcrc, incoming_command.crc); */
+	//debug(RPT_INFO, "%s: wrong CheckSum: computed %04x / real %04x",
+  	//	__FUNCTION__, testcrc, in->crc);
 	return(TRY_AGAIN);
 }
 
 
+#ifdef DEBUG
 /*
  * This is a debugging function.
  * It should be removed or compiled in conditionally.
@@ -571,4 +665,4 @@ print_packet(COMMAND_PACKET *packet)
 
 	fprintf(stderr, " ] %02x %02x .\n", packet->crc & 0xFF, (packet->crc >> 8) & 0xFF);
 }
-
+#endif /* DEBUG */
