@@ -31,22 +31,26 @@
 
 #define DEFAULT_DEVICE		"/dev/lcd"
 #define DEFAULT_BACKLIGHT	1
-#define DEFAULT_BRIGHTNESS	200
+#define DEFAULT_BRIGHTNESS	700
 
 #define DEFAULT_CELL_WIDTH_1602		5
 #define DEFAULT_CELL_WIDTH_12232	6
+#define DEFAULT_CELL_WIDTH_12832	6
 #define DEFAULT_CELL_WIDTH		DEFAULT_CELL_WIDTH_12232
 
 #define DEFAULT_CELL_HEIGHT_1602	8
 #define DEFAULT_CELL_HEIGHT_12232	8
+#define DEFAULT_CELL_HEIGHT_12832	8
 #define DEFAULT_CELL_HEIGHT		DEFAULT_CELL_HEIGHT_12232
 
 #define DEFAULT_SPEED_1602	19200
 #define DEFAULT_SPEED_12232	19200
+#define DEFAULT_SPEED_12832	19200
 #define DEFAULT_SPEED		DEFAULT_SPEED_12232
 
-#define	DEFAULT_SIZE_1602	"16x2"
-#define	DEFAULT_SIZE_12232	"20x4"
+#define DEFAULT_SIZE_1602	"16x2"
+#define DEFAULT_SIZE_12232	"20x4"
+#define DEFAULT_SIZE_12832	"21x4"
 #define DEFAULT_SIZE		DEFAULT_SIZE_12232
 
 
