@@ -1,3 +1,8 @@
+/** \file server/drivers/EyeboxOne.c
+ * LCDd \c EyeboxOne driver for the display used in the Eyebox ONE server
+ * by RightVision.
+ */
+
 /*  This is the LCDproc driver for EyeboxOne devices
 
     This code is based on MtxOrb driver (William Ferrell and Scott Scriven) 
@@ -61,17 +66,18 @@
 #define LED_ORANGE 2
 #define LED_GREEN 3
 
-typedef struct p {
+/** private data for the \c EyeBoxOne driver */
+typedef struct EyeboxOne_private_data {
 	int def[9];
 	int use[9];
-	int backlight_state;	/* static data from EyeboxOne_backlight */
-	int width;
-	int height;
-	char *framebuf;		/* Frame buffer */
-	char *old;		/* Current on screen frame buffer */
-	int widthBYheight;	/* Avoid computing width * height frequently */
-	int clear;		/* Control when the LCD is cleared */
-	int fd;			/* The LCD file descriptor */
+	int backlight_state;	/**< static data from EyeboxOne_backlight */
+	int width;		/**< display width in characters */
+	int height;		/**< display height in characters */
+	char *framebuf;		/**< frame buffer */
+	char *old;		/**< current on screen frame buffer */
+	int widthBYheight;	/**< avoid computing width * height frequently */
+	int clear;		/**< control when the LCD is cleared */
+	int fd;			/**< display file descriptor */
 	int backlightenabled;
 	int cursorenabled;
 	char left_key;
@@ -81,9 +87,9 @@ typedef struct p {
 	char enter_key;
 	char escape_key;
 	int keypad_test_mode;
-	int cellwidth;
-	int cellheight;
-	char info[255]; 
+	int cellwidth;		/**< character cell width */
+	int cellheight;		/**< character cell height */
+	char info[255]; 	/**< info string contents */
 	/*	LibData *libdata;		*/
 	/* Private Data of the new library: Work in progress */
 } PrivateData;
@@ -99,7 +105,8 @@ static void EyeboxOne_use_bar(int fd, int bar, int level);
 static void EyeboxOne_use_led(int fd, int led, int color);
 
 /* Parse one key from the configfile */
-static char EyeboxOne_parse_keypad_setting (Driver *drvthis, char * keyname, char default_value)
+static char
+EyeboxOne_parse_keypad_setting (Driver *drvthis, char * keyname, char default_value)
 {
 	char return_val = 0;
 	const char *s;
@@ -120,7 +127,7 @@ static char EyeboxOne_parse_keypad_setting (Driver *drvthis, char * keyname, cha
  *
  * Called to initialize driver settings
  */
-	MODULE_EXPORT int
+MODULE_EXPORT int
 EyeboxOne_init (Driver *drvthis)
 {
 	struct termios portset;
@@ -217,7 +224,7 @@ EyeboxOne_init (Driver *drvthis)
 
 	/* keypad test mode? */
 	if (drvthis->config_get_bool(drvthis->name, "keypad_test_mode", 0, 0)) {
-		fprintf( stdout, "EyeBO: Entering keypad test mode...\n");
+		fprintf(stdout, "EyeBO: Entering keypad test mode...\n");
 		p->keypad_test_mode = 1;
 		stay_in_foreground = 1;
 	}
@@ -230,27 +237,27 @@ EyeboxOne_init (Driver *drvthis)
 
 		/* left_key */
 		p->left_key = EyeboxOne_parse_keypad_setting(drvthis, "LeftKey", EYEBOXONE_DEFAULT_Left);
-		report(RPT_DEBUG, "%s: Using \"%c\" as Leftkey.", drvthis->name, p->left_key);
+		report(RPT_DEBUG, "%s: Using \"%c\" as Leftkey", drvthis->name, p->left_key);
 
 		/* right_key */
 		p->right_key = EyeboxOne_parse_keypad_setting(drvthis, "RightKey", EYEBOXONE_DEFAULT_Right);
-		report(RPT_DEBUG, "%s: Using \"%c\" as RightKey.", drvthis->name, p->right_key);
+		report(RPT_DEBUG, "%s: Using \"%c\" as RightKey", drvthis->name, p->right_key);
 
 		/* up_key */
 		p->up_key = EyeboxOne_parse_keypad_setting(drvthis, "UpKey", EYEBOXONE_DEFAULT_Up);
-		report(RPT_DEBUG, "%s: Using \"%c\" as UpKey.", drvthis->name, p->up_key);
+		report(RPT_DEBUG, "%s: Using \"%c\" as UpKey", drvthis->name, p->up_key);
 
 		/* down_key */
 		p->down_key = EyeboxOne_parse_keypad_setting(drvthis, "DownKey", EYEBOXONE_DEFAULT_Down);
-		report(RPT_DEBUG, "%s: Using \"%c\" as DownKey.", drvthis->name, p->down_key);
+		report(RPT_DEBUG, "%s: Using \"%c\" as DownKey", drvthis->name, p->down_key);
 
 		/* right_key */
 		p->enter_key = EyeboxOne_parse_keypad_setting(drvthis, "EnterKey", EYEBOXONE_DEFAULT_Enter);
-		report(RPT_DEBUG, "%s: Using \"%c\" as EnterKey.", drvthis->name, p->enter_key);
+		report(RPT_DEBUG, "%s: Using \"%c\" as EnterKey", drvthis->name, p->enter_key);
 
 		/* escape_key */
 		p->escape_key = EyeboxOne_parse_keypad_setting(drvthis, "EscapeKey", EYEBOXONE_DEFAULT_Escape);
-		report(RPT_DEBUG, "%s: Using \"%c\" as EscapeKey.", drvthis->name, p->escape_key);
+		report(RPT_DEBUG, "%s: Using \"%c\" as EscapeKey", drvthis->name, p->escape_key);
 
 	}
 	/* End of config file parsing*/
@@ -301,7 +308,7 @@ EyeboxOne_init (Driver *drvthis)
 
 	report(RPT_DEBUG, "%s: init() done", drvthis->name);
 
-	return 1;
+	return 0;
 }
 
 #define ValidX(x) if ((x) > p->width) { (x) = p->width; } else (x) = (x) < 1 ? 1 : (x);
@@ -311,7 +318,9 @@ EyeboxOne_init (Driver *drvthis)
 /********************
  * Use bars
  */
-static void EyeboxOne_use_bar(int fd, int bar, int level){
+static void
+EyeboxOne_use_bar(int fd, int bar, int level)
+{
 	char buffer[16];
 
 	if(bar > 2 || bar < 1)
@@ -328,7 +337,9 @@ static void EyeboxOne_use_bar(int fd, int bar, int level){
 /********************
  * Use leds
  */
-static void EyeboxOne_use_led(int fd, int led, int color){
+static void
+EyeboxOne_use_led(int fd, int led, int color)
+{
 	char buffer[16];
 	int a,b;
 
@@ -365,7 +376,7 @@ static void EyeboxOne_use_led(int fd, int led, int color){
 /******************************
  * Clear the screen (the frame buffer)
  */
-	MODULE_EXPORT void
+MODULE_EXPORT void
 EyeboxOne_clear (Driver *drvthis)
 {
 	PrivateData * p = drvthis->private_data;
@@ -380,7 +391,7 @@ EyeboxOne_clear (Driver *drvthis)
 /******************************
  * Clean-up
  */
-	MODULE_EXPORT void
+MODULE_EXPORT void
 EyeboxOne_close (Driver *drvthis)
 {
 	PrivateData * p = drvthis->private_data;
@@ -410,7 +421,7 @@ EyeboxOne_close (Driver *drvthis)
 /******************************
  * Returns the display width
  */
-	MODULE_EXPORT int
+MODULE_EXPORT int
 EyeboxOne_width (Driver *drvthis)
 {
 	PrivateData * p = drvthis->private_data;
@@ -421,7 +432,7 @@ EyeboxOne_width (Driver *drvthis)
 /******************************
  * Returns the display height
  */
-	MODULE_EXPORT int
+MODULE_EXPORT int
 EyeboxOne_height (Driver *drvthis)
 {
 	PrivateData * p = drvthis->private_data;
@@ -432,7 +443,7 @@ EyeboxOne_height (Driver *drvthis)
 /******************************
  * Display a string at x,y
  */
-	MODULE_EXPORT void
+MODULE_EXPORT void
 EyeboxOne_string (Driver *drvthis, int x, int y, const char string[])
 {
 	int offset, siz;
@@ -471,7 +482,7 @@ EyeboxOne_string (Driver *drvthis, int x, int y, const char string[])
 /******************************
  * Send what we have to the hardware
  */
-	MODULE_EXPORT void
+MODULE_EXPORT void
 EyeboxOne_flush (Driver *drvthis)
 {
 	char out[12];
@@ -520,7 +531,7 @@ EyeboxOne_flush (Driver *drvthis)
  * Prints a character on the lcd display, at position (x,y).
  * The upper-left is (1,1), and the lower right should be (20,4).
  */
-	MODULE_EXPORT void
+MODULE_EXPORT void
 EyeboxOne_chr (Driver *drvthis, int x, int y, char c)
 {
 	int offset;
@@ -540,11 +551,7 @@ EyeboxOne_chr (Driver *drvthis, int x, int y, char c)
 /******************************
  * Sets the backlight on or off 
  */
-
-#define BACKLIGHT_OFF 0
-#define BACKLIGHT_ON 1
-
-	MODULE_EXPORT void
+MODULE_EXPORT void
 EyeboxOne_backlight (Driver *drvthis, int on)
 {
 	PrivateData * p = drvthis->private_data;
@@ -581,7 +588,7 @@ EyeboxOne_backlight (Driver *drvthis, int on)
 /********************** 
  * Toggle cursor on/off
  */
-	static void
+static void
 EyeboxOne_showcursor (Driver *drvthis, int on)
 {
 	PrivateData * p = drvthis->private_data;
@@ -599,7 +606,7 @@ EyeboxOne_showcursor (Driver *drvthis, int on)
  * returns a string for the function characters from the keypad...
  * (A-Z) on success, 0 on failure...
  */
-	MODULE_EXPORT const char *
+MODULE_EXPORT const char *
 EyeboxOne_get_key (Driver *drvthis)
 {
 	PrivateData * p = drvthis->private_data;
@@ -652,7 +659,7 @@ EyeboxOne_get_key (Driver *drvthis)
  * Returns string with general information about the display
  * I think lcd is able to tell us more, but I can't find how...
  */
-	MODULE_EXPORT const char *
+MODULE_EXPORT const char *
 EyeboxOne_get_info (Driver *drvthis)
 {
 	PrivateData * p = drvthis->private_data;
