@@ -1,13 +1,16 @@
-/*  This is the LCDproc driver for Matrix Orbital devices
-    (http://www.matrixorbital.com)
-
-    For the Matrix Orbital LCD* LKD* VFD* and VKD* displays
-
-    Applicable Data Sheets:
-    - http://www.matrixorbital.ca/manuals/
-
-    NOTE: GLK displays have a different driver.
+/** \file server/drivers/MtxOrb.c
+ * LCDd \c MtxOrb driver for Matrix Orbital character mode displays.
+ *
+ * This driver supports the LCD*, LKD*, VFD*, and VKD* series of
+ * character mode displays by Matrix Orbital (http://www.matrixorbital.com).
+ * 
+ * Data sheets for the displays this driver supports can be found at
+ * http://www.matrixorbital.ca/manuals/
+ *
+ * \note The GLK series of graphical displays is supported by the \c glk driver.
+ */
 
+/*
     Copyright (C) 1999, William Ferrell and Scott Scriven
 		  2001, Andre Breiler
 		  2001, Philip Pokorny
@@ -41,6 +44,7 @@
 #include <termios.h>
 #include <fcntl.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
 #include <syslog.h>
 #include <ctype.h>
@@ -114,8 +118,9 @@ typedef enum {
 } CGmode;
 
 
-typedef struct {
-	int fd;			/* The LCD file descriptor */
+/** private data for the \c MtxOrb driver */
+typedef struct MtxOrb_private_data {
+	int fd;			/**< LCD file descriptor */
 	
 	/* dimensions */
 	int width, height;
@@ -128,10 +133,11 @@ typedef struct {
 	/* defineable characters */
 	CGmode ccmode;
 
-	int output_state;	/* static data from MtxOrb_output */
-	int contrast;		/* static data from set/get_contrast */
+	int output_state;	/**< current output state */
+	int contrast;		/**< current contrast */
 	int brightness;
 	int offbrightness;
+	int adjustable_backlight;
 
 	MtxOrb_type_type MtxOrb_type;
 
@@ -140,9 +146,65 @@ typedef struct {
 	int keys;
 	int keypad_test_mode;
 
-	char info[255];		/* static data from MtxOrb_get_info */
+	char info[255];		/**< info string contents */
 } PrivateData;
 
+/** List of known Matrix Orbital modules */
+static const MtxOrbModuleEntry modulelist[] = {
+	/* model, name, flags */
+	{ 0x01, "LCD0821", 0 },
+	{ 0x03, "LCD2021", 0 },
+	{ 0x04, "LCD1641", 0 },
+	{ 0x05, "LCD2041", 0 },	/* VK202-25 ? */
+	{ 0x06, "LCD4021", 0 },
+	{ 0x07, "LCD4041", 0 },
+	{ 0x08, "LK202-25", 0 },
+	{ 0x09, "LK204-25", 0 },
+	{ 0x0A, "LK404-55", 0 },
+	{ 0x0B, "VFD2021", 0 },
+	{ 0x0C, "VFD2041", 0 },
+	{ 0x0D, "VFD4021", 0 },
+	{ 0x0E, "VK202-25", 0 },
+	{ 0x0F, "VK204-25", 0 },
+	{ 0x10, "GLC12232", 0 },
+	{ 0x11, "GLC12864", 0 },
+	{ 0x12, "GLC128128", 0 },	/* ??? */
+	{ 0x13, "GLC24064", 0 },
+	{ 0x14, "GLK12864-25", 0 },	/* ??? */
+	{ 0x15, "GLK24064-25", 0 },
+	{ 0x21, "GLK128128-25", 0 },	/* ??? */
+	{ 0x22, "GLK12232-25-WBL", 0 },
+	{ 0x24, "GLK12232-25-SM", 0 },
+	{ 0x31, "LK404-AT", 0 },
+	{ 0x32, "VFD1621", 0 },	/* MOS-AV-162A ? */
+	{ 0x33, "LK402-12", 0 },
+	{ 0x34, "LK162-12", 0 },
+	{ 0x35, "LK204-25PC", 0 },
+	{ 0x36, "LK202-24-USB", 0 },
+	{ 0x37, "VK202-24-USB", 0 },
+	{ 0x38, "LK204-24-USB", 0 },
+	{ 0x39, "VK204-24-USB", 0 },
+	{ 0x3A, "PK162-12", 0 },
+	{ 0x3B, "VK162-12", 0 },
+	{ 0x3C, "MOS-AP-162A", 0 },
+	{ 0x3D, "PK202-25", 0 },
+	{ 0x3E, "MOS-AL-162A", 0 },
+	{ 0x40, "MOS-AV-202A", 0 },
+	{ 0x41, "MOS-AP-202A", 0 },
+	{ 0x42, "PK202-24-USB", 0 },
+	{ 0x43, "MOS-AL-082", 0 },
+	{ 0x44, "MOS-AL-204", 0 },
+	{ 0x45, "MOS-AV-204", 0 },
+	{ 0x46, "MOS-AL-402", 0 },
+	{ 0x47, "MOS-AV-402", 0 },
+	{ 0x48, "LK082-12", 0 },
+	{ 0x49, "VK402-12", 0 },
+	{ 0x4A, "VK404-55", 0 },
+	{ 0x4B, "LK402-25", 0 },
+	{ 0x4C, "VK402-25", 0 },
+	{ 0x00, NULL, 0 },
+};
+
 
 /* Vars for the server core */
 MODULE_EXPORT char *api_version = API_VERSION;
@@ -157,31 +219,11 @@ static void MtxOrb_cursorblink(Driver *drvthis, int on);
 static void MtxOrb_cursor_goto(Driver *drvthis, int x, int y);
 
 
-/* Parse one key from the configfile */
-static char
-MtxOrb_parse_keypad_setting (Driver *drvthis, char *keyname, char default_value)
-{
-	char return_val = 0;
-	const char *s;
-	char buf[255];
-
-	s = drvthis->config_get_string(drvthis->name, keyname, 0, NULL);
-	if (s != NULL) {
-		strncpy(buf, s, sizeof(buf));
-		buf[sizeof(buf)-1] = '\0';
-		return_val = buf[0];
-	} else {
-		return_val = default_value;
-	}
-	return return_val;
-}
-
-
 /**
  * Initialize the driver.
  * \param drvthis  Pointer to driver structure.
- * \retval 0   Success.
- * \retval <0  Error.
+ * \retval 0       Success.
+ * \retval <0      Error.
  */
 MODULE_EXPORT int
 MtxOrb_init (Driver *drvthis)
@@ -196,7 +238,7 @@ MtxOrb_init (Driver *drvthis)
 
         PrivateData *p;
 
-	/* Alocate and store private data */
+	/* Allocate and store private data */
         p = (PrivateData *) malloc(sizeof(PrivateData));
 	if (p == NULL)
 	        return -1;
@@ -249,6 +291,11 @@ MtxOrb_init (Driver *drvthis)
 	}
 	p->contrast = tmp;
 
+	/* Does it have an adjustable backlight */
+	tmp = drvthis->config_get_bool(drvthis->name, "hasAdjustableBacklight", 0, DEFAULT_ADJ_BACKLIGHT);
+	debug(RPT_INFO, "%s: hasAdjustableBacklight is '%d'", __FUNCTION__, tmp);
+	p->adjustable_backlight = tmp;
+	
 	/* Which backlight brightness */
 	tmp = drvthis->config_get_int(drvthis->name, "Brightness", 0, DEFAULT_BRIGHTNESS);
 	debug(RPT_INFO, "%s: Brightness (in config) is '%d'", __FUNCTION__, tmp);
@@ -311,7 +358,7 @@ MtxOrb_init (Driver *drvthis)
 
 	/* keypad test mode? */
 	if (drvthis->config_get_bool(drvthis->name, "keypad_test_mode", 0, 0)) {
-		fprintf( stdout, "MtxOrb: Entering keypad test mode...\n");
+		fprintf(stdout, "MtxOrb: Entering keypad test mode...\n");
 		p->keypad_test_mode = 1;
 		stay_in_foreground = 1;
 	}
@@ -409,6 +456,8 @@ MtxOrb_init (Driver *drvthis)
 	MtxOrb_cursorblink(drvthis, DEFAULT_CURSORBLINK);
 	MtxOrb_set_contrast(drvthis, p->contrast);
 	MtxOrb_backlight(drvthis, DEFAULT_BACKLIGHT);
+	MtxOrb_get_info(drvthis);
+	report(RPT_INFO, "Display detected: %s", p->info);
 
 	report(RPT_DEBUG, "%s: init() done", drvthis->name);
 
@@ -447,7 +496,7 @@ MtxOrb_close (Driver *drvthis)
 /**
  * Return the display width in characters.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of characters the display is wide.
+ * \return         Number of characters the display is wide.
  */
 MODULE_EXPORT int
 MtxOrb_width (Driver *drvthis)
@@ -461,7 +510,7 @@ MtxOrb_width (Driver *drvthis)
 /**
  * Return the display height in characters.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of characters the display is high.
+ * \return         Number of characters the display is high.
  */
 MODULE_EXPORT int
 MtxOrb_height (Driver *drvthis)
@@ -475,10 +524,10 @@ MtxOrb_height (Driver *drvthis)
 /**
  * Return the width of a character in pixels.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of pixel columns a character cell is wide.
+ * \return         Number of pixel columns a character cell is wide.
  */
 MODULE_EXPORT int
-MtxcOrb_cellwidth (Driver *drvthis)
+MtxOrb_cellwidth (Driver *drvthis)
 {
 	PrivateData *p = drvthis->private_data;
 
@@ -489,10 +538,10 @@ MtxcOrb_cellwidth (Driver *drvthis)
 /**
  * Return the height of a character in pixels.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of pixel lines a character cell is high.
+ * \return         Number of pixel lines a character cell is high.
  */
 MODULE_EXPORT int
-MtxcOrb_cellheight (Driver *drvthis)
+MtxOrb_cellheight (Driver *drvthis)
 {
 	PrivateData *p = drvthis->private_data;
 
@@ -569,6 +618,11 @@ MtxOrb_flush (Driver *drvthis)
 		unsigned char *sp = p->framebuf + (i * p->width);
 		unsigned char *sq = p->backingstore + (i * p->width);
 
+		// set  pointers to end of the line in frame buffer & backing store
+		unsigned char *ep = sp + (p->width - 1);
+		unsigned char *eq = sq + (p->width - 1);
+		int length = 0;
+
 		debug(RPT_DEBUG, "Framebuf: '%.*s'", p->width, sp);
 		debug(RPT_DEBUG, "Backingstore: '%.*s'", p->width, sq);
 
@@ -577,11 +631,6 @@ MtxOrb_flush (Driver *drvthis)
 		 * - leave out leading and trailing parts that are identical
 		 */
 
-		// set  pointers to end of the line in frame buffer & backing store
-		unsigned char *ep = sp + (p->width - 1);
-		unsigned char *eq = sq + (p->width - 1);
-		int length = 0;
-
 		// skip over leading identical portions of the line
 		for (j = 0; (sp <= ep) && (*sp == *sq); sp++, sq++, j++)
 			;
@@ -645,7 +694,7 @@ MtxOrb_chr (Driver *drvthis, int x, int y, char c)
  * This is only the locally stored contrast, the contrast value
  * cannot be retrieved from the LCD.
  * \param drvthis  Pointer to driver structure.
- * \return  Stored contrast in promille.
+ * \return         Stored contrast in promille.
  */
 
 MODULE_EXPORT int
@@ -662,8 +711,8 @@ MtxOrb_get_contrast (Driver *drvthis)
  * Whis only works on LCD displays where the
  * HW supports values from 0 to 255 with 140 looking OK.
  * Is it better to use the brightness for VFD/VKD displays ?
- * \param drvthis  Pointer to driver structure.
- * \param promille New contrast value in promille.
+ * \param drvthis   Pointer to driver structure.
+ * \param promille  New contrast value in promille.
  */
 MODULE_EXPORT void
 MtxOrb_set_contrast (Driver *drvthis, int promille)
@@ -698,7 +747,7 @@ MtxOrb_set_contrast (Driver *drvthis, int promille)
  * Retrieve brightness.
  * \param drvthis  Pointer to driver structure.
  * \param state    Brightness state (on/off) for which we want the value.
- * \return Stored brightness in promille.
+ * \return         Stored brightness in promille.
  */
 MODULE_EXPORT int
 MtxOrb_get_brightness(Driver *drvthis, int state)
@@ -727,12 +776,11 @@ MtxOrb_set_brightness(Driver *drvthis, int state, int promille)
 	/* store the software value since there is no get */
 	if (state == BACKLIGHT_ON) {
 		p->brightness = promille;
-		MtxOrb_backlight(drvthis, BACKLIGHT_ON);
 	}
 	else {
 		p->offbrightness = promille;
-		MtxOrb_backlight(drvthis, BACKLIGHT_OFF);
 	}
+	MtxOrb_backlight(drvthis, state);
 }
 
 
@@ -745,28 +793,42 @@ MODULE_EXPORT void
 MtxOrb_backlight (Driver *drvthis, int on)
 {
 	PrivateData *p = drvthis->private_data;
-	int promille = (on == BACKLIGHT_ON)
-			     ? p->brightness
-			     : p->offbrightness;
 
-	if (IS_VKD_DISPLAY) {
-		unsigned char out[5] = { '\xFE', '\x89', 0 };
+	if (p->adjustable_backlight) {
+	
+		int promille = (on == BACKLIGHT_ON)
+				     ? p->brightness
+				     : p->offbrightness;
 
-		/* map range [0, 1000] -> [0, 3] that the hardware understands */
-		out[2] = (unsigned char) ((long) promille * 3 / 1000);
+		if (IS_VKD_DISPLAY) {
+			unsigned char out[5] = { '\xFE', 'Y', 0 };
 
-		write(p->fd, out, 3);
+			/* map range [0, 1000] -> [0, 3] that the hardware understands */
+			out[2] = (unsigned char) ((long) promille * 3 / 1000);
+
+			write(p->fd, out, 3);
+		}
+		else {
+			unsigned char out[5] = { '\xFE', '\x99', 0 };
+
+			/* map range [0, 1000] -> [0, 255] that the hardware understands */
+			out[2] = (unsigned char) ((long) promille * 255 / 1000);
+
+			write(p->fd, out, 3);
+		}
+
+		debug(RPT_DEBUG, "MtxOrb: changed brightness to %d", promille);
 	}
 	else {
-		unsigned char out[5] = { '\xFE', '\x99', 0 };
-
-		/* map range [0, 1000] -> [0, 255] that the hardware understands */
-		out[2] = (unsigned char) ((long) promille * 255 / 1000);
-
-		write(p->fd, out, 3);
+		if (on == BACKLIGHT_ON) {
+			unsigned char out[4] = { '\xFE', 'B', '\0', 0};
+			write(p->fd, out, 3);
+		}
+		else {
+			unsigned char out[4] = { '\xFE', 'F', 0};
+			write(p->fd, out, 2);
+		}
 	}
-
-	debug(RPT_DEBUG, "MtxOrb: changed brightness to %d", promille);
 }
 
 
@@ -810,7 +872,7 @@ MtxOrb_output (Driver *drvthis, int state)
 
 
 /**
- * Clear the LCD using ints hardware command
+ * Clear the LCD using its hardware command
  * \param drvthis  Pointer to driver structure.
  */
 static void
@@ -904,14 +966,14 @@ MtxOrb_cursor_goto(Driver *drvthis, int x, int y)
 /**
  * Provide general information about the LCD/VFD display.
  * \param drvthis  Pointer to driver structure.
- * \return  Constant string with information.
+ * \return         Constant string with information.
  */
 MODULE_EXPORT const char *
 MtxOrb_get_info (Driver *drvthis)
 {
-	char in = 0;
-	char tmp[255], buf[64];
-	/* int i = 0; */
+	char tmp[10], buf[255];
+	int model = 0;
+	int i;
         PrivateData *p = drvthis->private_data;
 
 	fd_set rfds;
@@ -922,115 +984,56 @@ MtxOrb_get_info (Driver *drvthis)
 	debug(RPT_DEBUG, "MtxOrb: get_info");
 
 	memset(p->info, '\0', sizeof(p->info));
-	strcpy(p->info, "Matrix Orbital Driver ");
-
-	/*
-	 * Read type of display
-	 */
-
-	write(p->fd, "\x0FE" "7", 2);
+	strcat(p->info, "Matrix Orbital, ");
 
 	/* Watch fd to see when it has input. */
 	FD_ZERO(&rfds);
 	FD_SET(p->fd, &rfds);
 
+	/*
+	 * Read type of display
+	 * In query for its type, the display return a single byte value.
+	 */
+	memset(tmp, '\0', sizeof(tmp));
+	write(p->fd, "\x0FE" "7", 2);
+
 	/* Wait the specified amount of time. */
 	tv.tv_sec = 0;		/* seconds */
 	tv.tv_usec = 500;	/* microseconds */
 
-/*	retval = select(p->fd+1, &rfds, NULL, NULL, &tv); */
-	retval = select(p->fd+1, &rfds, NULL, NULL, NULL);
+	retval = select(p->fd+1, &rfds, NULL, NULL, &tv); 
 
 	if (retval) {
-		if (read (p->fd, &in, 1) < 0) {
-			syslog(LOG_WARNING, "MatrixOrbital driver: unable to read data");
-		} else {
-			switch (in) {
-				case '\x01': strcat(p->info, "LCD0821 "); break;
-				case '\x03': strcat(p->info, "LCD2021 "); break;
-				case '\x04': strcat(p->info, "LCD1641 "); break;
-				case '\x05': strcat(p->info, "LCD2041 "); break;	/* VK202-25 ? */
-				case '\x06': strcat(p->info, "LCD4021 "); break;
-				case '\x07': strcat(p->info, "LCD4041 "); break;
-				case '\x08': strcat(p->info, "LK202-25 "); break;
-				case '\x09': strcat(p->info, "LK204-25 "); break;
-				case '\x0A': strcat(p->info, "LK404-55 "); break;
-				case '\x0B': strcat(p->info, "VFD2021 "); break;
-				case '\x0C': strcat(p->info, "VFD2041 "); break;
-				case '\x0D': strcat(p->info, "VFD4021 "); break;
-				case '\x0E': strcat(p->info, "VK202-25 "); break;
-				case '\x0F': strcat(p->info, "VK204-25 "); break;
-				case '\x10': strcat(p->info, "GLC12232 "); break;
-				case '\x11': strcat(p->info, "GLC12864 "); break;
-				case '\x12': strcat(p->info, "GLC128128 "); break;	/* ??? */
-				case '\x13': strcat(p->info, "GLC24064 "); break;
-				case '\x14': strcat(p->info, "GLK12864-25 "); break;	/* ??? */
-				case '\x15': strcat(p->info, "GLK24064-25 "); break;
-				case '\x21': strcat(p->info, "GLK128128-25 "); break;	/* ??? */
-				case '\x22': strcat(p->info, "GLK12232-25-WBL "); break;
-				case '\x24': strcat(p->info, "GLK12232-25-SM "); break;
-				case '\x31': strcat(p->info, "LK404-AT "); break;
-				case '\x32': strcat(p->info, "VFD1621 "); break;	/* MOS-AV-162A ? */
-				case '\x33': strcat(p->info, "LK402-12 "); break;
-				case '\x34': strcat(p->info, "LK162-12 "); break;
-				case '\x35': strcat(p->info, "LK204-25PC "); break;
-				case '\x36': strcat(p->info, "LK202-24-USB "); break;
-				case '\x37': strcat(p->info, "VK202-24-USB "); break;
-				case '\x38': strcat(p->info, "LK204-24-USB "); break;
-				case '\x39': strcat(p->info, "VK204-24-USB "); break;
-				case '\x3A': strcat(p->info, "PK162-12 "); break;
-				case '\x3B': strcat(p->info, "VK162-12 "); break;
-				case '\x3C': strcat(p->info, "MOS-AP-162A "); break;
-				case '\x3D': strcat(p->info, "PK202-25 "); break;
-				case '\x3E': strcat(p->info, "MOS-AL-162A "); break;
-				case '\x40': strcat(p->info, "MOS-AV-202A "); break;
-				case '\x41': strcat(p->info, "MOS-AP-202A "); break;
-				case '\x42': strcat(p->info, "PK202-24-USB "); break;
-				case '\x43': strcat(p->info, "MOS-AL-082 "); break;
-				case '\x44': strcat(p->info, "MOS-AL-204 "); break;
-				case '\x45': strcat(p->info, "MOS-AV-204 "); break;
-				case '\x46': strcat(p->info, "MOS-AL-402 "); break;
-				case '\x47': strcat(p->info, "MOS-AV-402 "); break;
-				case '\x48': strcat(p->info, "LK082-12 "); break;
-				case '\x49': strcat(p->info, "VK402-12 "); break;
-				case '\x4A': strcat(p->info, "VK404-55 "); break;
-				case '\x4B': strcat(p->info, "LK402-25 "); break;
-				case '\x4C': strcat(p->info, "VK402-25 "); break;
-				default: /*snprintf(tmp, sizeof(tmp), "Unknown (%X) ", in); strcat(info, tmp); */
-					     break;
+		if (read(p->fd, &tmp, 1) < 0)
+			report(RPT_WARNING, "%s: unable to read data", drvthis->name);
+		else {
+			for (i = 0; modulelist[i].model != 0; i++) {
+				if (modulelist[i].model == tmp[0]) {
+					snprintf(buf, sizeof(buf), "Model: %s, ", modulelist[i].name);
+					strncat(p->info, buf, sizeof(p->info) - strlen(p->info) - 1);
+					model = (int) tmp[0];
+					break;
+				}
 			}
 		}
-	} else
-		syslog(LOG_WARNING, "MatrixOrbital driver: unable to read device type");
+	}
+	else
+		report(RPT_WARNING, "%s: unable to read device type", drvthis->name);
 
-	/*
-	 * Read serial number of display
-	 */
-
-	memset(tmp, '\0', sizeof(tmp));
-	write(p->fd, "\x0FE" "5", 2);
-
-	/* Wait the specified amount of time. */
-	tv.tv_sec = 0;		/* seconds */
-	tv.tv_usec = 500;	/* microseconds */
-
-/*	retval = select(p->fd+1, &rfds, NULL, NULL, &tv); */
-	retval = select(p->fd+1, &rfds, NULL, NULL, NULL);
-
-	if (retval) {
-		if (read(p->fd, &tmp, 2) < 0) {
-			syslog(LOG_WARNING, "MatrixOrbital driver: unable to read data");
-		} else {
-			snprintf(buf, sizeof(buf), "Serial No: %ld ", (long int) tmp);
-			strcat(p->info, buf);
-		}
-	} else
-		syslog(LOG_WARNING, "MatrixOrbital driver: unable to read device serial number");
+	if (model == 0) {
+		snprintf(buf, sizeof(buf), "Unknown model (0x%02x), ", (int) tmp[0]);
+		strncat(p->info, buf, sizeof(p->info) - strlen(p->info) - 1);
+	}
 
 	/*
 	 * Read firmware revision number
+	 * TODO: check the format of a real display, because mine does not
+	 * return anything.
+	 * It is assumed that the display returns a two byte value giving
+	 * the major/minor version number.
+	 * NOTE: This is just a guess as the manual doesn't describe
+	 * the returned format.
 	 */
-
 	memset(tmp, '\0', sizeof(tmp));
 	write(p->fd, "\x0FE" "6", 2);
 
@@ -1038,19 +1041,46 @@ MtxOrb_get_info (Driver *drvthis)
 	tv.tv_sec = 0;		/* seconds */
 	tv.tv_usec = 500;	/* microseconds */
 
-/*	retval = select(p->fd+1, &rfds, NULL, NULL, &tv); */
-	retval = select(p->fd+1, &rfds, NULL, NULL, NULL);
+	retval = select(p->fd+1, &rfds, NULL, NULL, &tv); 
 
 	if (retval) {
-		if (read(p->fd, &tmp, 2) < 0) {
-			syslog(LOG_WARNING, "MatrixOrbital driver: unable to read data");
-		} else {
-			snprintf(buf, sizeof(buf), "Firmware Rev. %ld ", (long int) tmp);
-			strcat(p->info, buf);
-		}
-	} else
-		syslog(LOG_WARNING, "MatrixOrbital driver: unable to read device firmware revision");
+		if (read(p->fd, &tmp, 2) < 0)
+			report(RPT_WARNING, "%s: unable to read data", drvthis->name);
+		
+	}
+	else
+		report(RPT_WARNING, "%s: unable to read device firmware revision", drvthis->name);
 
+	snprintf(buf, sizeof(buf), "Firmware Rev.: 0x%02x 0x%02x, ", tmp[0], tmp[1]);
+	strncat(p->info, buf, sizeof(p->info) - strlen(p->info) - 1);
+
+	/*
+	 * Read serial number of display
+	 * It is assumed that the display returns a two byte value giving
+	 * the serial number.
+	 * NOTE: This is just a guess as the manual doesn't describe
+	 * the returned format, but for setting the serial number two bytes
+	 * have to be supplied, so it is likely the display will return those.
+	 */
+	memset(tmp, '\0', sizeof(tmp));
+	write(p->fd, "\x0FE" "5", 2);
+
+	/* Wait the specified amount of time. */
+	tv.tv_sec = 0;		/* seconds */
+	tv.tv_usec = 500;	/* microseconds */
+
+	retval = select(p->fd+1, &rfds, NULL, NULL, &tv); 
+
+	if (retval) {
+		if (read(p->fd, &tmp, 2) < 0)
+			report(RPT_WARNING, "%s: unable to read data", drvthis->name);
+	}
+	else
+		report(RPT_WARNING, "%s: unable to read device serial number", drvthis->name);
+
+	snprintf(buf, sizeof(buf), "Serial No: 0x%02x 0x%02x", tmp[0], tmp[1]);
+	strncat(p->info, buf, sizeof(p->info) - strlen(p->info) - 1);
+		
 	return p->info;
 }
 
@@ -1124,7 +1154,7 @@ MtxOrb_hbar (Driver *drvthis, int x, int y, int len, int promille, int options)
 
 		for (i = 1; i <= p->cellwidth; i++) {
 			// fill pixel columns from left to right.
-			memset(hBar, 0xFF & ~((1 << (p->cellwidth - i)) - 1), sizeof(hBar)-1);
+			memset(hBar, 0xFF & ~((1 << (p->cellwidth - i)) - 1), sizeof(hBar));
 			MtxOrb_set_char(drvthis, i, hBar);
 		}
 	}
@@ -1169,7 +1199,7 @@ MtxOrb_num (Driver *drvthis, int x, int num)
 /**
  * Get total number of custom characters available.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of custom characters (always NUM_CCs).
+ * \return         Number of custom characters (always NUM_CCs).
  */
 MODULE_EXPORT int
 MtxOrb_get_free_chars (Driver *drvthis)
@@ -1193,7 +1223,7 @@ MODULE_EXPORT void
 MtxOrb_set_char (Driver *drvthis, int n, unsigned char *dat)
 {
 	PrivateData *p = drvthis->private_data;
-	unsigned char out[12] = { '\xFE', 'N', 0, 0,0,0,0,0,0,0,0 };;
+	unsigned char out[12] = { '\xFE', 'N', 0, 0,0,0,0,0,0,0,0 };
 	unsigned char mask = (1 << p->cellwidth) - 1;
 	int row;
 
@@ -1217,7 +1247,8 @@ MtxOrb_set_char (Driver *drvthis, int n, unsigned char *dat)
  * \param x        Horizontal character position (column).
  * \param y        Vertical character position (row).
  * \param icon     synbolic value representing the icon.
- * \return  Information whether the icon is handled here or needs to be handled by the server core.
+ * \retval 0       Icon has been successfully defined/written.
+ * \retval <0      Server core shall define/write the icon.
  */
 MODULE_EXPORT int
 MtxOrb_icon (Driver *drvthis, int x, int y, int icon)
@@ -1424,7 +1455,8 @@ MtxOrb_cursor (Driver *drvthis, int x, int y, int state)
 /**
  * Get key from the LCD/VFD.
  * \param drvthis  Pointer to driver structure.
- * \return  String representation of the key.
+ * \return         String representation of the key.
+ *                 \c NULL if nothing available / unmapped key.
  *
  * \todo Recover the code for I2C connectivity to MtxOrb
  * and don't query the LCD if it does not support keypad.
