@@ -31,16 +31,7 @@
 #define LCD2USB_GET_CTRL	(LCD2USB_GET | (2<<3))
 #define LCD2USB_GET_RESERVED1	(LCD2USB_GET | (3<<3))
 
-#define DEFAULT_CONTRAST	300
-#define DEFAULT_BRIGHTNESS	600
-#define DEFAULT_OFFBRIGHTNESS	300
-
 // initialise this particular driver
 int hd_init_lcd2usb(Driver *drvthis);
 
-void lcd2usb_HD44780_senddata(PrivateData *p, unsigned char displayID, unsigned char flags, unsigned char ch);
-void lcd2usb_HD44780_backlight(PrivateData *p, unsigned char state);
-unsigned char lcd2usb_HD44780_scankeypad(PrivateData *p);
-void lcd2usb_HD44780_close(PrivateData *p);
-
 #endif
