@@ -6,6 +6,6 @@
 
 // initialise this particular driver, args is probably not used but keep
 // for consistency
-int hd_init_usblcd (HD44780_functions * hd44780_functions, lcd_logical_driver * driver, char *args, unsigned int port);
+int hd_init_usblcd(Driver *drvthis);
 
 #endif
