@@ -1,10 +1,13 @@
-/*
- * "Winamp" 8-bit driver module for Hitachi HD44780 based LCD displays.
- * The LCD is operated in it's 8 bit-mode to be connected to a single
- * PC parallel port.
+/** \file server/drivers/hd44780-winamp.c
+ * \c winamp connection type of \c hd44780 driver for Hitachi HD44780 based LCD displays.
  *
+ * The LCD is operated in its 8 bit-mode to be connected to a single PC parallel port.
+ */
+
+/*
  * Copyright (c)  1999 Andrew McMeikan <andrewm@engineer.com>
- *		  modular driver 1999-2000 Benjamin Tse <blt@Comports.com>
+ *		  1999-2000 Benjamin Tse <blt@Comports.com>
+ *                  - modular driver
  *
  *		  Modified July 2000 by Charles Steinkuehler for enhanced
  *		  performance and reduced CPU usage
@@ -14,6 +17,9 @@
  *                2001 Joris Robijn <joris@robijn.net>
  *                  - Keypad support
  *
+ * This file is released under the GNU General Public License. Refer to the
+ * COPYING file distributed with this package.
+ *
  * The connections are:
  * printer port   LCD
  * D0 (2)	  D0 (7)
@@ -53,18 +59,14 @@
  * PAPEREND (12)  X2
  * SELIN  (13)    X3
  * nFAULT (15)    X4
- *
- * Created modular driver Dec 1999, Benjamin Tse <blt@Comports.com>
- *
- * This file is released under the GNU General Public License. Refer to the
- * COPYING file distributed with this package.
  */
 
 #include "hd44780-winamp.h"
 #include "hd44780-low.h"
 #include "lpt-port.h"
-
 #include "port.h"
+#include "report.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -80,6 +82,8 @@ void lcdwinamp_HD44780_backlight(PrivateData *p, unsigned char state);
 unsigned char lcdwinamp_HD44780_readkeypad(PrivateData *p, unsigned int YData);
 void lcdwinamp_HD44780_output(PrivateData *p, int data);
 
+// Compile time mapping of control lines
+// For expert users only!
 #define EN1	STRB
 #define EN2	SEL
 #define EN3	LF
@@ -90,13 +94,36 @@ void lcdwinamp_HD44780_output(PrivateData *p, int data);
 
 static const unsigned char EnMask[] = { EN1, EN2, EN3 };
 
-// initialise the driver
+
+/**
+ * Initialize the driver.
+ * \param drvthis  Pointer to driver structure.
+ * \retval 0       Success.
+ * \retval -1      Error.
+ */
 int
 hd_init_winamp(Driver *drvthis)
 {
 	PrivateData *p = (PrivateData*) drvthis->private_data;
 	HD44780_functions *hd44780_functions = p->hd44780_functions;
 
+	// Safety check against common configuration errors
+	if (p->numDisplays == 2) {
+		if (p->have_backlight && (EnMask[1] == BL)) {
+			report(RPT_ERR, "hd_init_winamp: backlight must be on different pin than 2nd controller");
+			report(RPT_ERR, "hd_init_winamp: please change connection mapping in hd44780-winamp.c");
+			return -1;
+		}
+		if (p->have_backlight && p->have_output) {
+			report(RPT_ERR, "hd_init_winamp: backlight and output not possible with 2 controllers");
+			return -1;
+		}
+	}
+	else if (p->numDisplays == 3 && (p->have_backlight || p->have_output)) {
+		report(RPT_ERR, "hd_init_winamp: backlight or output not possible with 3 controllers");
+		return -1;
+	}
+
 	// Reserve the port registers
 	port_access_multiple(p->port,3);
 
@@ -123,7 +150,14 @@ hd_init_winamp(Driver *drvthis)
 	return 0;
 }
 
-// lcdwinamp_HD44780_senddata
+
+/**
+ * Send data or commands to the display.
+ * \param p          Pointer to driver's private data structure.
+ * \param displayID  ID of the display (or 0 for all) to send data to.
+ * \param flags      Defines whether to end a command or data.
+ * \param ch         The value to send.
+ */
 void
 lcdwinamp_HD44780_senddata(PrivateData *p, unsigned char displayID, unsigned char flags, unsigned char ch)
 {
@@ -137,8 +171,8 @@ lcdwinamp_HD44780_senddata(PrivateData *p, unsigned char displayID, unsigned cha
 	portControl |= p->backlight_bit;
 
 	if (displayID == 0)
-		enableLines = EnMask[0] 
-		| ((p->have_backlight) ? 0 : EnMask[1])
+		enableLines = EnMask[0]
+		| ((p->numDisplays >= 2) ? EnMask[1] : 0)
 		| ((p->numDisplays == 3) ? EnMask[2] : 0);
 	else
 		enableLines = EnMask[displayID - 1];
@@ -168,13 +202,26 @@ lcdwinamp_HD44780_senddata(PrivateData *p, unsigned char displayID, unsigned cha
 	// 10 nS data hold time provided by the length of ISA write for EN
 }
 
+
+/**
+ * Turn display backlight on or off.
+ * \param p      Pointer to driver's private data structure.
+ * \param state  New backlight status.
+ */
 void lcdwinamp_HD44780_backlight(PrivateData *p, unsigned char state)
 {
-	p->backlight_bit = (state?0:nSEL);
+	p->backlight_bit = (state ? 0 : BL);
 
 	port_out(p->port + 2, p->backlight_bit ^ OUTMASK);
 }
 
+
+/**
+ * Read keypress.
+ * \param p      Pointer to driver's private data structure.
+ * \param YData  Bitmap of rows / lines to enable.
+ * \return       Bitmap of the pressed keys.
+ */
 unsigned char lcdwinamp_HD44780_readkeypad(PrivateData *p, unsigned int YData)
 {
 	unsigned char readval;
@@ -196,11 +243,17 @@ unsigned char lcdwinamp_HD44780_readkeypad(PrivateData *p, unsigned int YData)
 		((readval & ACK) / ACK )) & ~p->stuckinputs;	/* pin 10 */
 }
 
+
+/**
+ * Set output port.
+ * \param p      Pointer to driver's private data structure.
+ * \param data   Value the output port shall be set to.
+ */
 void lcdwinamp_HD44780_output(PrivateData *p, int data)
 {
 	// Setup data bus
 	port_out(p->port, data);
-	// Strobe the latch (374 latches on rising edge, 3/574 on trailing.  No matter)
+	// Strobe the latch (374 latches on rising edge, 373 on trailing. No matter)
 	port_out(p->port + 2, (LE | p->backlight_bit) ^ OUTMASK);
         if (p->delayBus) p->hd44780_functions->uPause(p, 1);
 	port_out(p->port + 2, (p->backlight_bit) ^ OUTMASK);
