@@ -40,23 +40,23 @@
 
 #include "lcd.h"
 
-MODULE_EXPORT int  MTC_S16209X_init(Driver * drvthis);
-MODULE_EXPORT void MTC_S16209X_close(Driver * drvthis);
+MODULE_EXPORT int  MTC_S16209X_init(Driver *drvthis);
+MODULE_EXPORT void MTC_S16209X_close(Driver *drvthis);
 MODULE_EXPORT int  MTC_S16209X_width (Driver *drvthis);
 MODULE_EXPORT int  MTC_S16209X_height (Driver *drvthis);
 MODULE_EXPORT int  MTC_S16209X_cellwidth (Driver *drvthis);
 MODULE_EXPORT int  MTC_S16209X_cellheight (Driver *drvthis);
-MODULE_EXPORT void MTC_S16209X_clear (Driver * drvthis);
-MODULE_EXPORT void MTC_S16209X_flush(Driver * drvthis);
-MODULE_EXPORT void MTC_S16209X_string (Driver * drvthis, int x, int y, char string[]);
-MODULE_EXPORT void MTC_S16209X_chr(Driver * drvthis, int x, int y, char c) ;
+MODULE_EXPORT void MTC_S16209X_clear (Driver *drvthis);
+MODULE_EXPORT void MTC_S16209X_flush(Driver *drvthis);
+MODULE_EXPORT void MTC_S16209X_string (Driver *drvthis, int x, int y, const char string[]);
+MODULE_EXPORT void MTC_S16209X_chr(Driver *drvthis, int x, int y, char c) ;
 
-MODULE_EXPORT void MTC_S16209X_vbar(Driver * drvthis, int x, int y, int len, int promille, int options);
-MODULE_EXPORT void MTC_S16209X_hbar(Driver * drvthis, int x, int y, int len, int promille, int options);
-MODULE_EXPORT int  MTC_S16209X_icon(Driver * drvthis, int x, int y, int icon);
+MODULE_EXPORT void MTC_S16209X_vbar(Driver *drvthis, int x, int y, int len, int promille, int options);
+MODULE_EXPORT void MTC_S16209X_hbar(Driver *drvthis, int x, int y, int len, int promille, int options);
+MODULE_EXPORT int  MTC_S16209X_icon(Driver *drvthis, int x, int y, int icon);
 
-MODULE_EXPORT void MTC_S16209X_set_char(Driver * drvthis, int n, char *dat);
+MODULE_EXPORT void MTC_S16209X_set_char(Driver *drvthis, int n, char *dat);
 
-MODULE_EXPORT void MTC_S16209X_backlight(Driver * drvthis, int on);
+MODULE_EXPORT void MTC_S16209X_backlight(Driver *drvthis, int on);
 
 #endif
