@@ -53,8 +53,9 @@ typedef enum {
     custom1 = 16
 } custom_type;
 
-typedef struct
-{
+
+/** private data for the \c pyramid driver */
+typedef struct pyramid_private_data {
     /* device io */
     int FD;
     char device[255];
@@ -84,7 +85,7 @@ typedef struct
     int C_state;
 
     /* led handling */
-    char led[7];
+    char led[8];
 
 } PrivateData;
 
