@@ -1,8 +1,13 @@
+/** \file server/drivers/serialPOS.c
+ * LCDd \c serialPOS driver for Point Of Sale ("POS") devices using
+ * various protocols.
+ */
+
 /* 	This is the LCDproc driver for Point Of Sale ("POS") devices using 
 	various protocols.  While it currently only supports AEDEX, 
 	it can be extended to provide support for many POS emulation types.
 
-	Copyright (C) 2006 Eric Pooch
+	Copyright (C) 2006, 2007 Eric Pooch
 
 	This driver is based on MtxOrb.c driver and is subject to its copyrights.
 	
@@ -27,7 +32,7 @@ init		Implemented.
 close		Implemented.
 width		Implemented.
 height		Implemented.
-clear		Implemented by space filling no custom char info.
+clear		Implemented.
 flush		Implemented.
 string		Implemented.
 chr		Implemented.
@@ -35,10 +40,10 @@ vbar		Implemented.
 hbar		Implemented.
 num		Implemented.
 heartbeat	Implemented.
-icon		NOT IMPLEMENTED: not part of POS protocol
-cursor		NOT IMPLEMENTED: not part of AEDEX protocol
-set_char	NOT IMPLEMENTED: not part of AEDEX protocol
-get_free_chars	Implemented.
+icon		NOT IMPLEMENTED: not part of any POS protocol
+cursor		Implemented.
+set_char	NOT IMPLEMENTED
+get_free_chars	Implemented, always returns 0.
 cellwidth	Implemented.
 cellheight	Implemented.
 get_contrast	NOT IMPLEMENTED: not part of AEDEX protocol
@@ -46,8 +51,8 @@ set_contrast	NOT IMPLEMENTED: not part of AEDEX protocol
 get_brightness	NOT IMPLEMENTED: not part of AEDEX protocol
 set_brightness	NOT IMPLEMENTED: not part of AEDEX protocol
 backlight	NOT IMPLEMENTED: not part of AEDEX protocol
-output		Not implemented.
-get_key		Not implemented, no keys.
+output		NOT IMPLEMENTED: not part of any POS protocol
+get_key		Implemented for devices using a pass-through serial port connected to an RS232 terminal or keyboard.
 get_info	Implemented.
 */
 
@@ -59,6 +64,7 @@ get_info	Implemented.
 #include <termios.h>
 #include <fcntl.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
 #include <syslog.h>
 #include <ctype.h>
@@ -102,17 +108,17 @@ get_info	Implemented.
 
 
 typedef enum {
-	POS_IEE = 0,
-	POS_AEDEX,
-	POS_Epson,
+	POS_IEE = 0,		
+	POS_AEDEX,				/* http://www.posguys.com/download/CD5220_Manual.pdf */
+	POS_Epson,				/* http://www.barcode-manufacturer.com/pdf/vfd_manual.pdf */
 	POS_Emax,
 	POS_IBM,
-	POS_LogicControls,
+	POS_LogicControls,		/* http://www.posguys.com/download/CD5220_Manual.pdf */
 	POS_Ultimate
 } POS_EmulationType;
 
-
-#define AEDEXPrefix "!#"
+#define AEDEXDefaultPrefix "!#"
+#define AEDEXPrefix "~`"
 
 typedef enum {
 	AEDEXLine1Display = 1,	/*	upper line display	*/
@@ -138,9 +144,9 @@ typedef enum {
 
 
 
-
-typedef struct {
-	int fd;			/* The LCD file descriptor */
+/** private data for the \c serialPOS driver */
+typedef struct serialPOS_private_data {
+	int fd;			/**< LCD file descriptor */
 	
 	/* dimensions */
 	int width, height;
@@ -157,10 +163,9 @@ typedef struct {
 	int hardwrap;
 	int hardscroll;
 
-	POS_EmulationType emulation_mode;	/* The emulation type  */
-	int output_state;	/* static data from serialPOS_output */
+	POS_EmulationType emulation_mode;	/**< emulation type */
 
-	char info[255];		/* static data from serialPOS_get_info */
+	char info[255];		/**< static data from serialPOS_get_info */
 } PrivateData;
 
 
@@ -170,6 +175,7 @@ MODULE_EXPORT int stay_in_foreground = 0;
 MODULE_EXPORT int supports_multiple = 1;
 MODULE_EXPORT char *symbol_prefix = "serialPOS_";
 
+static void serialPOS_hardware_init(Driver *drvthis);
 static void serialPOS_hardware_clear(Driver *drvthis);
 static void serialPOS_linewrap(Driver *drvthis, int on);
 static void serialPOS_autoscroll(Driver *drvthis, int on);
@@ -200,8 +206,8 @@ serialPOS_parse_keypad_setting (Driver *drvthis, char *keyname, char default_val
 /**
  * Initialize the driver.
  * \param drvthis  Pointer to driver structure.
- * \retval 0   Success.
- * \retval <0  Error.
+ * \retval 0       Success.
+ * \retval <0      Error.
  */
 MODULE_EXPORT int
 serialPOS_init (Driver *drvthis)
@@ -234,8 +240,6 @@ serialPOS_init (Driver *drvthis)
 	p->framebuf = NULL;
 	p->backingstore = NULL;
 
-	p->output_state = -1;	/* static data from serialPOS_output */
-
 	p->hardwrap = 0;
 	p->hardscroll = 0;
 
@@ -360,7 +364,7 @@ serialPOS_init (Driver *drvthis)
 	memset(p->backingstore, ' ', p->width * p->height);
 
 	/* set initial LCD configuration */
-	serialPOS_hardware_clear(drvthis);
+	serialPOS_hardware_init(drvthis);
 	serialPOS_linewrap(drvthis, DEFAULT_LINEWRAP);
 	serialPOS_autoscroll(drvthis, DEFAULT_AUTOSCROLL);
 
@@ -401,7 +405,7 @@ serialPOS_close (Driver *drvthis)
 /**
  * Return the display width in characters.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of characters the display is wide.
+ * \return         Number of characters the display is wide.
  */
 MODULE_EXPORT int
 serialPOS_width (Driver *drvthis)
@@ -415,7 +419,7 @@ serialPOS_width (Driver *drvthis)
 /**
  * Return the display height in characters.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of characters the display is high.
+ * \return         Number of characters the display is high.
  */
 MODULE_EXPORT int
 serialPOS_height (Driver *drvthis)
@@ -427,12 +431,24 @@ serialPOS_height (Driver *drvthis)
 
 
 /**
- * Return the width of a character in pixels.
+ * Get total number of custom characters available.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of pixel columns a character cell is wide.
+ * \return         Number of custom characters (always \c 0 ).
  */
 MODULE_EXPORT int
-MtxcOrb_cellwidth (Driver *drvthis)
+serialPOS_get_free_chars (Driver *drvthis)
+{
+	return 0;
+}
+
+
+/**
+ * Return the width of a character in pixels.
+ * \param drvthis  Pointer to driver structure.
+ * \return         Number of pixel columns a character cell is wide.
+ */
+MODULE_EXPORT int
+serialPOS_cellwidth (Driver *drvthis)
 {
 	PrivateData *p = drvthis->private_data;
 
@@ -443,10 +459,10 @@ MtxcOrb_cellwidth (Driver *drvthis)
 /**
  * Return the height of a character in pixels.
  * \param drvthis  Pointer to driver structure.
- * \return  Number of pixel lines a character cell is high.
+ * \return         Number of pixel lines a character cell is high.
  */
 MODULE_EXPORT int
-MtxcOrb_cellheight (Driver *drvthis)
+serialPOS_cellheight (Driver *drvthis)
 {
 	PrivateData *p = drvthis->private_data;
 
@@ -519,7 +535,6 @@ serialPOS_flush (Driver *drvthis)
 		unsigned char *sq = p->backingstore + (i * p->width);
 
 		unsigned int length = p->width+5;
-
 		char out[length];
 
 		debug(RPT_DEBUG, "Framebuf: '%.*s'", p->width, sp);
@@ -538,26 +553,36 @@ serialPOS_flush (Driver *drvthis)
 
 		/* there are differences, ... */
 		report(RPT_DEBUG, "%s: l=%d string='%.*s'", __FUNCTION__, i, p->width, sp);
-
-		if (p->emulation_mode == POS_AEDEX) {
-			int command = i+1;
-			if ( i == 0 && p->hardscroll == 1 )
-				command = AEDEXContinuousScroll;
+	
+		switch ((int)p->emulation_mode) {
+			case POS_AEDEX:	{
+			
+				int command = i+1;
+				if ( i == 0 && p->hardscroll == 1 )
+					command = AEDEXContinuousScroll;
 				
-			snprintf(out, length, "%s%d%.*s%c", AEDEXPrefix, command, p->width, sp, 13);
-			debug(RPT_DEBUG, "%s%d%.*s%c", AEDEXPrefix, command, p->width, sp, 13);
+				snprintf(out, length, "%s%d%.*s%c", AEDEXPrefix, command, p->width, sp, 13);
+				debug(RPT_DEBUG, "%s%d%.*s%c", AEDEXPrefix, command, p->width, sp, 13);
+				break;
+			}
+			
+			default:
+				/* Send to correct line, then write the data */
+				serialPOS_cursor_goto(drvthis, 1, i+1);
+				length = p->width+1;
+				snprintf(out, length, "%s", sp);
+				break;
 		}
+		debug(RPT_DEBUG, "%s%c", out);
 
- 		write(p->fd, out, sizeof(out));
+ 		write(p->fd, out, length);
 
 		modified++;
-
 	}
 
 	if (modified)
 		memcpy(p->backingstore, p->framebuf, p->width * p->height);
 
-	
 	report(RPT_DEBUG, "serialPOS: frame buffer flushed");
 }
 
@@ -579,32 +604,47 @@ serialPOS_chr (Driver *drvthis, int x, int y, char c)
 	x--;
 	y--;
 
-	/* The # character might interfere with the AEDEX command set */
-	if (p->emulation_mode == POS_AEDEX && c == '#')
-		c = '%';
-
 	if ((x >= 0) && (y >= 0) && (x < p->width) && (y < p->height))
 			p->framebuf[(y * p->width) + x] = c;
 
 	report(RPT_DEBUG, "writing character %02X to position (%d,%d)", c, x, y);
 }
 
-
 /**
- * Set output port(s).
- * Displays with keypad have 6 outputs whereas the others only have one.
+ * Send init codes for the display.
  * \param drvthis  Pointer to driver structure.
- * \param state    Integer with bits representing port states.
  */
-#ifdef NOTUSED
-MODULE_EXPORT void
-serialPOS_output (Driver *drvthis, int state)
+static void
+serialPOS_hardware_init(Driver *drvthis)
 {
-	/* I don't get what this does.  Many POS displays have a pass-through function. */
 	PrivateData *p = drvthis->private_data;
 
+			
+	switch ((int)p->emulation_mode) {
+		case POS_AEDEX:	{
+			/* The # character might interfere with the AEDEX command set, 
+			   so change the AEDEX attention code.
+			*/
+			unsigned int length = 8;
+			char out[length];
+
+			snprintf(out, length, "%s%d%s%c", AEDEXDefaultPrefix, AEDEXChangeCode, AEDEXPrefix, 13);
+			write(p->fd, out, length);
+
+			break;
+		}
+		case POS_Epson:
+			write(p->fd, "\x1B\x40", 2);
+			break;
+			
+		case POS_LogicControls:
+			write(p->fd, "\x11", 1);
+			break;
+			
+		default:
+			return;
+	}
 }
-#endif
 
 
 /**
@@ -614,17 +654,31 @@ serialPOS_output (Driver *drvthis, int state)
 static void
 serialPOS_hardware_clear (Driver *drvthis)
 {
-	/* There is no hardware clear in the POS command sets,
-		but most have an easy software clear. */
-	/*PrivateData *p = drvthis->private_data;
-
-	char out[6];
-
-	if (p->emulation_mode == POS_AEDEX) {
-		snprintf(out, 6, "%s%c %d", AEDEXPrefix, AEDEXAllLineDisplay, 13);
+	/* Do a hardware clear, if possible. */
+	PrivateData *p = drvthis->private_data;
+	
+	switch ((int)p->emulation_mode) {
+		case POS_AEDEX: {
+			unsigned int length = 7;
+			char out[length];
+			
+			snprintf(out, length, "%s%c %d", AEDEXPrefix, AEDEXAllLineDisplay, 13);
+			write(p->fd, out, length);
+			break;
+		}
+		
+		case POS_Epson:
+			write(p->fd, "\x0C", 1);
+			break;
+			
+		case POS_LogicControls:
+			write(p->fd, "\x1F", 1);
+			break;
+		
+		default:
+			return;
 	}
 	
- 	write(p->fd, out, 5);*/
 	debug(RPT_DEBUG, "serialPOS: cleared LCD");
 }
 
@@ -685,14 +739,35 @@ serialPOS_cursorblink (Driver *drvthis, int on)
 static void
 serialPOS_cursor_goto(Driver *drvthis, int x, int y)
 {
-	//PrivateData *p = drvthis->private_data;
+	PrivateData *p = drvthis->private_data;
+	unsigned int length = 8;
+	char out[length];
+	
+	switch ((int)p->emulation_mode) {
+		case POS_Epson:	
+			length = 7;
+			
+			snprintf(out, length, "%c%c%02d%02d", 0x1F, 0x24, x, y);
+			break;
+			
+		case POS_LogicControls:
+			length = 4;
+			
+			snprintf(out, length, "%c%02d", 0x10, (x-1+((y-1)*(p->width))));
+			break;
+		
+		default:
+			return;
+	}
+	
+	write(p->fd, out, length);
 }
 
 
 /**
- * Provide general information about the LCD/VFD display.
+ * Provide general information about the POS display.
  * \param drvthis  Pointer to driver structure.
- * \return  Constant string with information.
+ * \return         Constant string with information.
  */
 MODULE_EXPORT const char *
 serialPOS_get_info (Driver *drvthis)
@@ -812,93 +887,12 @@ serialPOS_hbar (Driver *drvthis, int x, int y, int len, int promille, int option
  * \param num      Character to write (0 - 10 with 10 representing ':')
  */
 MODULE_EXPORT void
-serialPOS_num (Driver *drvthis, int x, int num)
+serialPOS_num(Driver * drvthis, int x, int num)
 {
-	PrivateData *p = drvthis->private_data;
-	int do_init = 0;
-
-	if ((num < 0) || (num > 10))
-		return;
-
-	if (p->ccmode != bignum) {
-		if (p->ccmode != standard) {
-			/* Not supported (yet) */
-			report(RPT_WARNING, "%s: num: cannot combine two modes using user-defined characters",
-					drvthis->name);
-			return;
-		}
-
-		p->ccmode = bignum;
-
-		do_init = 1;
-	}
-
-	// Lib_adv_bignum does everything needed to show the bignumbers.
-	//lib_adv_bignum(drvthis, x, num, 0, do_init);
+        // Lib_adv_bignum does everything needed to show the bignumbers.
+        lib_adv_bignum(drvthis, x, num, 0, 0);
 }
 
-#ifdef NOTUSED
-/**
- * Get total number of custom characters available.
- * \param drvthis  Pointer to driver structure.
- * \return  Number of custom characters (always NUM_CCs).
- */
-MODULE_EXPORT int
-serialPOS_get_free_chars (Driver *drvthis)
-{
-	//PrivateData *p = drvthis->private_data;
-
-	return NUM_CCs;
-}
-
-
-/**
- * Define a custom character and write it to the LCD.
- * \param drvthis  Pointer to driver structure.
- * \param n	Custom character to define [0 - (NUM_CCs-1)].
- * \param dat      Array of 8(=cellheight) bytes, each representing a pixel row
- *		 starting from the top to bottom.
- *		 The bits in each byte represent the pixels where the LSB
- *		 (least significant bit) is the rightmost pixel in each pixel row.
- */
-MODULE_EXPORT void
-serialPOS_set_char (Driver *drvthis, int n, unsigned char *dat)
-{
-	PrivateData *p = drvthis->private_data;
-	unsigned char mask = (1 << p->cellwidth) - 1;
-	int row;
-
-	if ((n < 0) || (n >= NUM_CCs))
-		return;
-	if (!dat)
-		return;
-
-	out[2] = n;	/* Custom char to define. xxx */
-
-	for (row = 0; row < p->cellheight; row++) {
-		out[row+3] = dat[row] & mask;
-	}
-	write(p->fd, out, 11);
-}
-
-
-/**
- * Place an icon on the screen.
- * \param drvthis  Pointer to driver structure.
- * \param x	Horizontal character position (column).
- * \param y	Vertical character position (row).
- * \param icon     synbolic value representing the icon.
- * \return  Information whether the icon is handled here or needs to be handled by the server core.
- */
-MODULE_EXPORT int
-serialPOS_icon (Driver *drvthis, int x, int y, int icon)
-{
-	//PrivateData *p = drvthis->private_data;
-
-	return 0;
-}
-
-
 /**
  * Set cursor position and state.
  * \param drvthis  Pointer to driver structure.
@@ -909,60 +903,103 @@ serialPOS_icon (Driver *drvthis, int x, int y, int icon)
 MODULE_EXPORT void 
 serialPOS_cursor (Driver *drvthis, int x, int y, int state)
 {
-	//hPrivateData *p = (PrivateData *) drvthis->private_data;
+	PrivateData *p = drvthis->private_data;
+	
+	switch ((int)p->emulation_mode) {
+		case POS_LogicControls:
+			switch (state) {
+				case CURSOR_OFF:        // no cursor
+					write(p->fd, "\x14", 1);
+					break;
+				case CURSOR_DEFAULT_ON: // blinking block
+					write(p->fd, "\x13", 1);
+					break;
+				default:
+					break;
+			}
+			break;
+		
+		default:
+			/* set cursor state not supported */
+			break;
+	}
 
-	/* set cursor state */
-	 //serialPOS_cursor_goto(drvthis, x, y);
+	/* set cursor position */
+	serialPOS_cursor_goto(drvthis, x, y);
 }
 
+
 /**
- * Get key from the LCD/VFD.
+ * Get key from a pass-through port of the POS display.
  * \param drvthis  Pointer to driver structure.
- * \return  String representation of the key.
- *
+ * \return         String representation of the key;
+ *                 \c NULL if nothing available / unmapped key.
  */
 MODULE_EXPORT const char *
 serialPOS_get_key (Driver *drvthis)
 {
 	PrivateData *p = drvthis->private_data;
-	char key = 0;
-	struct pollfd fds[1];
+	int ret;
+	char buf;
+	const char *key = NULL;
+	static struct timeval selectTimeout = { 0, 0 };
+	fd_set fdset;
 
-	/* don't query the keyboard if there are no mapped keys; see \todo above */
-	if ((p->keys == 0) && (!p->keypad_test_mode))
+	FD_ZERO(&fdset);
+	FD_SET(p->fd, &fdset);
+
+	if ((ret = select(FD_SETSIZE, &fdset, NULL, NULL, &selectTimeout)) < 0) {
+		report(RPT_DEBUG, "%s: get_key: select() failed (%s)",
+				drvthis->name, strerror(errno));
+		return NULL;
+	}
+	if (!ret) {
+		FD_SET(p->fd, &fdset);
+		return NULL;
+	}
+
+	if (!FD_ISSET(p->fd, &fdset))
 		return NULL;
 
-	/* poll for data or return */
-	fds[0].fd = p->fd;
-	fds[0].events = POLLIN;
-	fds[0].revents = 0;
-	poll(fds,1,0);
-	if (fds[0].revents == 0)
+	if ((ret = read(p->fd, &buf, 1)) < 0) {
+		report(RPT_DEBUG, "%s: get_key: read() failed (%s)",
+				drvthis->name, strerror(errno));
 		return NULL;
+	}
 
-	(void) read(p->fd, &key, 1);
-	report(RPT_DEBUG, "%s: get_key: key 0x%02X", drvthis->name, key);
+	if (ret == 1) {
 
-	if (key == '\0')
-		return NULL;
-
-	if (!p->keypad_test_mode) {
-		/* we assume standard key mapping here */
-		if ((key >= 'A') && (key <= 'A' + MAX_KEY_MAP)) {
-			return p->keymap[key-'A'];
-		}
-		else {
-			report(RPT_INFO, "%s: Untreated key 0x%02X", drvthis->name, key);
+		switch (buf) {
+		case 13:
+			key = "Enter";
+			break;
+		case 65:
+			key = "Up";
+			break;
+		case 66:
+			key = "Down";
+			break;
+		case 67:
+			key = "Right";
+			break;
+		case 68:
+			key = "Left";
+			break;
+		case 8:
+			key = "Escape";
+			break;
+		default:
+			report(RPT_DEBUG, "%s get_key: illegal key 0x%02X", 
+					drvthis->name, buf);
 			return NULL;
 		}
+
+		report(RPT_DEBUG, "%s: get_key: returns %s", drvthis->name, key);
+		return key;
 	}
-	else {
-		fprintf(stdout, "serialPOS: Received character %c\n", key);
-		fprintf(stdout, "serialPOS: Press another key of your device.\n");
-	}
+
 	return NULL;
 }
 
-#endif
 
 /* EOF */
