@@ -1,14 +1,4 @@
-/*
- * menuitem.h
- * This file is part of LCDd, the lcdproc server.
- *
- * This file is released under the GNU General Public License. Refer to the
- * COPYING file distributed with this package.
- *
- * Copyright (c) 1999, William Ferrell, Scott Scriven
- *               2004, F5 Networks, Inc. - IP-address input
- *               2005, Peter Marschall - error checks, ...
- *
+/** \file server/menuitem.h
  * Defines all the menuitem data and actions.
  *
  * There are a few different menuitems:
@@ -24,6 +14,16 @@
  * One menuitem is in a different file: Menu data is in menu,h.
  */
 
+/* This file is part of LCDd, the lcdproc server.
+ *
+ * This file is released under the GNU General Public License.
+ * Refer to the COPYING file distributed with this package.
+ *
+ * Copyright (c) 1999, William Ferrell, Scott Scriven
+ *               2004, F5 Networks, Inc. - IP-address input
+ *               2005, Peter Marschall - error checks, ...
+ */
+
 #ifndef MENUITEM_H
 #define MENUITEM_H
 
@@ -36,6 +36,7 @@
 #endif
 
 #define max(a,b) (((a) > (b)) ? (a) : (b))
+#define min(a,b) (((a) < (b)) ? (a) : (b))
 
 /*********************************************************************
  * Data definitions of the menustuff
@@ -43,14 +44,14 @@
 
 /** These values are used in the function tables in menuitem.c ! */
 typedef enum MenuItemType {
-	MENUITEM_MENU = 0,
-	MENUITEM_ACTION = 1,
+	MENUITEM_MENU     = 0,
+	MENUITEM_ACTION   = 1,
 	MENUITEM_CHECKBOX = 2,
-	MENUITEM_RING = 3,
-	MENUITEM_SLIDER = 4,
-	MENUITEM_NUMERIC = 5,
-	MENUITEM_ALPHA = 6,
-	MENUITEM_IP = 7,
+	MENUITEM_RING     = 3,
+	MENUITEM_SLIDER   = 4,
+	MENUITEM_NUMERIC  = 5,
+	MENUITEM_ALPHA    = 6,
+	MENUITEM_IP       = 7,
         NUM_ITEMTYPES = 8
 } MenuItemType;
 
@@ -58,21 +59,22 @@ typedef enum CheckboxValue {
 	CHECKBOX_OFF = 0, CHECKBOX_ON, CHECKBOX_GRAY
 } CheckboxValue;
 
-/** Recognized input token codes */
+/** Recognized input token codes: they need to be bit values */
 typedef enum MenuToken {
-	MENUTOKEN_MENU,
-	MENUTOKEN_ENTER,
-	MENUTOKEN_UP,
-	MENUTOKEN_DOWN,
-	MENUTOKEN_LEFT,
-	MENUTOKEN_RIGHT,
-	MENUTOKEN_OTHER
+	MENUTOKEN_NONE  = 0x0000,	/**< no key */
+	MENUTOKEN_MENU  = 0x0001,	/**< MenuKey */
+	MENUTOKEN_ENTER = 0x0002,	/**< EnterKey */
+	MENUTOKEN_UP    = 0x0004,	/**< UpKey */
+	MENUTOKEN_DOWN  = 0x0008,	/**< DownKey */
+	MENUTOKEN_LEFT  = 0x0010,	/**< LeftKey */
+	MENUTOKEN_RIGHT = 0x0020,	/**< RightKey */
+	MENUTOKEN_OTHER = 0x1000	/**< any other key */
 } MenuToken;
 
 /** Return codes from an input handler */
 typedef enum MenuResult {
 	MENURESULT_ERROR = -1,	/**< Something has gone wrong */
-	MENURESULT_NONE = 0,	/**< Token handled OK, no extra action */
+	MENURESULT_NONE  = 0,	/**< Token handled OK, no extra action */
 	MENURESULT_ENTER,	/**< Token handled OK, enter the selected
 				 * menuitem now */
 	MENURESULT_CLOSE,	/**< Token handled OK, close the current
@@ -87,16 +89,16 @@ typedef enum MenuResult {
 /** Events caused by a menuitem */
 typedef enum MenuEventType {
 	MENUEVENT_SELECT = 0,	/**< Item has been selected
-				(action chosen) */
+				 * (action chosen) */
 	MENUEVENT_UPDATE = 1,	/**< Item has been modified
-				(checkbox, numeric, alphanumeric) */
-	MENUEVENT_PLUS = 2,	/**< Item has been modified in positive direction
-				 (slider moved) */
-	MENUEVENT_MINUS = 3,	/**< Item has been modified in negative direction
-				(slider moved) */
-	MENUEVENT_ENTER = 4,	/**< Menu has been entered */
-	MENUEVENT_LEAVE = 5,    /**< Menu has been left */
-        NUM_EVENTTYPES = 6
+				 * (checkbox, numeric, alphanumeric) */
+	MENUEVENT_PLUS   = 2,	/**< Item has been modified in positive direction
+				 * (slider moved) */
+	MENUEVENT_MINUS  = 3,	/**< Item has been modified in negative direction
+				 * (slider moved) */
+	MENUEVENT_ENTER  = 4,	/**< Menu has been entered */
+	MENUEVENT_LEAVE  = 5,	/**< Menu has been left */
+	NUM_EVENTTYPES   = 6
 } MenuEventType;
 
 #define MenuEventFunc(f) int (f) (struct MenuItem *item, MenuEventType event)
@@ -310,7 +312,7 @@ void menuitem_update_screen(MenuItem *item, Screen *s);
 /** Does something with the given input.
  * key is only used if token is MENUTOKEN_OTHER.
  */
-MenuResult menuitem_process_input(MenuItem *item, MenuToken token, const char *key, bool extended);
+MenuResult menuitem_process_input(MenuItem *item, MenuToken token, const char *key, unsigned int keymask);
 
 /** returns the Client that owns the MenuItem. item must not be null */
 Client *menuitem_get_client(MenuItem *item);
