@@ -1,19 +1,18 @@
-/*
- * parse.c
- * This file is part of LCDd, the lcdproc server.
- *
- * This file is released under the GNU General Public License. Refer to the
- * COPYING file distributed with this package.
- *
- * Copyright (c) 1999, William Ferrell, Scott Scriven
- *
- *
+/** \file server/parse.c
  * Handles input commands from clients, by splitting strings into tokens
  * and passing arguments to the appropriate handler.
  *
  * It works much like a command line.  Only the first token is used to
  * determine what function to call.
+ */
+ 
+/* This file is part of LCDd, the lcdproc server.
  *
+ * This file is released under the GNU General Public License.
+ * Refer to the COPYING file distributed with this package.
+ *
+ * Copyright (c) 1999, William Ferrell, Scott Scriven
+ *               2008, Peter Marschall
  */
 
 #include <stdlib.h>
@@ -26,7 +25,7 @@
 #include "clients.h"
 #include "commands/command_list.h"
 #include "parse.h"
-
+#include "sock.h"
 
 #define MAX_ARGUMENTS 40
 
@@ -229,6 +228,11 @@ parse_all_client_messages(void)
 		for (str = client_get_message(c); str != NULL; str = client_get_message(c)) {
 			parse_message(str, c);
 			free(str);
+
+			if (c->state == GONE) {
+				sock_destroy_client_socket(c);
+				break;
+			}
 		}
 	}
 	return 0;
