@@ -5,7 +5,7 @@
 #include <stdio.h>
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 /* DEBUGGING
